from functools import lru_cache
import logging
import os
import re
import time
from typing import Optional, Union
import warnings

import requests
from requests.adapters import HTTPAdapter
import urllib3
from urllib3.util.retry import Retry
from requests_ratelimiter import LimiterAdapter
from pyrate_limiter import SQLiteBucket

import pynsee
from pynsee.utils._get_credentials import _get_credentials_from_configfile
from pynsee.utils._create_insee_folder import _create_insee_folder
from pynsee.constants import SIRENE_KEY, HTTPS_PROXY_KEY, HTTP_PROXY_KEY

logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def _warn_env_credentials(var) -> None:
    logger.warning(
        "Existing environment variable %s used, instead of locally "
        "saved credentials",
        var,
    )


def get_env_case_insensitive(x: str):
    """Fetch an environment variable (case insentitive)"""
    return os.environ.get(x, os.environ.get(x.upper()))


class PynseeAPISession(requests.Session):
    """
    Session class used throughout pynsee for http(s) queries. This session
    object uses a specific set of config values, use
    `help(PynseeAPISession.__init__)` for more informations.
    """

    INSEE_API_CODES = {
        # 200:"Opération réussie",
        # 301:"Moved Permanently" -> r.headers['location']
        400: "Bad Request",
        401: "Unauthorized : token missing",
        403: "Forbidden : missing subscription to API",  #
        # 404: "Not Found : no results available",
        406: "Not acceptable : incorrect 'Accept' header",
        413: "Too many results, query must be splitted",
        414: "Request-URI Too Long",
        # Unused (managed through a specific if/else), kept for memory purpose
        # 429: "Too Many Requests : allocated quota overloaded",
        500: "Internal Server Error ",
        503: "Service Unavailable",
    }

    def __init__(
        self,
        sirene_key: Optional[str] = None,
        http_proxy: Optional[str] = None,
        https_proxy: Optional[str] = None,
    ):
        """
        Generates a PynseeAPISession used to perform http(s) requests in pynsee

        Arguments are loaded from this order :
            * explicit argument;
            * if not given, from os.environ
            * if still not found, will try to get the previous valid
              configuration.

        **This function should only be loaded with arguments through
        pynsee.utils.init_conn**.

        See :func:`~pynsee.utils.init_conn` to create a new configuration.

        Parameters
        ----------
        sirene_key : str, optional
            Token generated by the API portal. Use this if you wish to access
            the SIRENE API and pynsee has not already been configured. The
            default is None.

        """

        super().__init__()
        self._mount_adapters()

        config = {
            HTTP_PROXY_KEY: http_proxy,
            HTTPS_PROXY_KEY: https_proxy,
            SIRENE_KEY: sirene_key,
        }

        stored_config = _get_credentials_from_configfile()

        for k, v in config.items():
            venv = get_env_case_insensitive(k)
            if v is None:
                # Use env variable if it exist, otherwise stored value.
                # Note: v might be set to the empty string for proxy config,
                # which is **not** the same as None
                if venv is not None:
                    config[k] = venv
                    _warn_env_credentials(k)
                else:
                    config[k] = stored_config.get(k)

        self.proxies.update(
            {
                "http": config[HTTP_PROXY_KEY],
                "https": config[HTTPS_PROXY_KEY],
            }
        )

        username = os.environ.get("USERNAME", "username")
        version = pynsee.__version__
        useragent = {"User-Agent": f"python_pynsee_{username}/{version}"}

        # Note : geoplatform seems to impose the "/version" to the user-agent
        self.headers.update(useragent)

        self.sirene_key = config[SIRENE_KEY]
        self.headers["X-INSEE-Api-Key-Integration"] = self.sirene_key

    def _mount_adapters(self):
        """
        Mount adapters for http queries :
            * default retries config for every http(s) requests
            * and with specific rate-limiting for each API called

        Those rate-limits are ensured with requests-ratelimiter and an SQLite
        backend, stored by default in pynsee's appdata folder. They are **NOT**
        (yet) adequate for any task shared between multiple machines.

        """

        # default retries adapter
        # 429 Too Many Requests (RFC 6585)
        # 502 Bad Gateway
        # 503 Service Unavailable
        # 504 Gateway Timeout
        retry_adapt = Retry(
            total=7, backoff_factor=1, status_forcelist=[429, 502, 503, 504]
        )
        adapter = HTTPAdapter(max_retries=retry_adapt)
        self.mount("http://", adapter)
        self.mount("https://", adapter)

        insee_folder = _create_insee_folder()
        rate_folder = os.path.join(insee_folder, "rate_limiter")

        os.makedirs(rate_folder, exist_ok=True)

        def kw_adapter(api: str):
            "set given arguments to configure an ratelimiter adapter"
            return {
                "max_retries": retry_adapt,
                "bucket_class": SQLiteBucket,
                "bucket_kwargs": {
                    "path": os.path.join(rate_folder, f"{api}.db"),
                    "isolation_level": "EXCLUSIVE",
                    "check_same_thread": False,
                },
            }

        # set custom adapters for each API, to allow a separate rate tracking
        # for each
        rates = {
            "sirene": {
                "url": "https://api.insee.fr/api-sirene",
                # 30 queries/min for SIRENE
                "rates": {"per_minute": 30},
            },
            "bdm": {
                "url": "https://api.insee.fr/series/BDM",
                # 30 queries/min for BDM: from documentation, though there is
                # no need of a token?!
                "rates": {"per_minute": 30},
            },
            "sdmx": {
                "url": "https://bdm.insee.fr",
                # not documented afik, but let's set it to the same rate as bdm
                "rates": {"per_minute": 30},
            },
            "localdata": {
                "url": "https://api.insee.fr/donnees-locales",
                # 30 queries/min for (old) localdata: from documentation,
                # though there is no need of a token?!
                "rates": {"per_minute": 30},
            },
            "melodi": {
                "url": "https://api.insee.fr/melodi",
                # 30 queries/min for melodi: from documentation, though there
                # is no need of a token?!
                "rates": {"per_minute": 30},
            },
            "metadata": {
                "url": "https://api.insee.fr/metadonnees/",
                # 10_000 queries/min for metadata (from subscription page)
                # but INSEE confirm this is a default value (corresponding
                # to the theoric coverage of simultaneous requests by the
                # portal). Let's go for a more realistic rate instead...
                "rates": {"per_minute": 600},
            },
            "nominatim": {
                "url": "https://nominatim.openstreetmap.org/",
                # https://operations.osmfoundation.org/policies/nominatim/
                "rates": {"per_second": 1},
            },
            # note : nothing found in data.geopf.fr's documentation ?
        }

        for api, config in rates.items():
            adapter = LimiterAdapter(**config["rates"], **kw_adapter(api))
            self.mount(config["url"], adapter)

        # add retry on 400 for geoplatform
        retry_adapt = Retry(
            total=7,
            backoff_factor=1,
            status_forcelist=[400],
        )
        adapter = HTTPAdapter(max_retries=retry_adapt)
        self.mount("https://data.geopf.fr", adapter)

    def request(
        self,
        method: str,
        url: str,
        timeout: Union[tuple, int] = (10, 15),
        raise_if_not_ok: bool = True,
        **kwargs,
    ) -> requests.Response:
        """
        Overwrite requests.Session's request. Allows to set specific timeouts
        and to raise exceptions if response is not ok. Also silences urllib's
        warnings on insecure requests being performed.

        Parameters
        ----------
        method : str
            Usually, "GET" or "POST".
        url : str
            URL to query.
        timeout : Union[tuple, int], optional
            timeout used for the query. See requests' documentation for more
            info. The default is (10, 15).
        raise_if_not_ok : bool, optional
            If set to True, a RequestException will automatically be raised if
            the response is not ok (= `status_code` < 400).
            The default is True.
        **kwargs :
            Any other kwargs are passed directly to requests.Session.request

        Raises
        ------
        RequestException
            If the requests fails (only if raise_if_not_ok is set to True).

        Returns
        -------
        response : requests.Response
            HTTP response from the requests package.

        """

        logger.info(url)
        with warnings.catch_warnings():
            warnings.simplefilter(
                "ignore", urllib3.exceptions.InsecureRequestWarning
            )
            response = super().request(method, url, timeout=timeout, **kwargs)
            if raise_if_not_ok and not response.ok:
                raise requests.exceptions.RequestException(
                    f"response was {response.status_code} for {url}",
                    response=response,
                )
        return response

    def request_insee(
        self,
        api_url: Optional[str] = None,
        sdmx_url: Optional[str] = None,
        file_format: str = "application/xml",
        print_msg: bool = True,
        raise_if_not_ok: bool = False,
    ) -> requests.Response:
        """
        Performs a query to INSEE, either through API or sdmx_url

        Parameters
        ----------
        api_url : Optional[str]
            URL to be queried on the API portal, optional. The default is None.
        sdmx_url : Optional[str]
            URL to be queried on the SDMX (Statistical Data and Metadata
            eXchange) webservice of INSEE, optional. The default is None.
        file_format : str, optional
            Which king of file to expect. This currently alters the output of
            INSEE's APIs. The default is "application/xml".
        print_msg : bool, optional
            If True, will log critical entries to warn of failures to query
            the APIs. The default is True.
        raise_if_not_ok : bool, optional
            See PynseeAPISession.request. The default is False.

        Raises
        ------
        RequestException
            In case sirene_key is missing for a call to SIRENE API.

        ValueError
            If neither api_url or sdmx_url have been set.

        Returns
        -------
        result : requests.Response
            HTTP response from the requests package.

        """

        try:
            if api_url and self._query_is_valid_sirene_call(api_url):
                if not self.sirene_key:
                    commands = "\n\ninit_conn(sirene_key='my_sirene_key')\n"
                    msg = (
                        "Sirene key is missing, please check your credentials "
                        "on portail-api.insee.fr !\n"
                        "Please do the following to use your "
                        f"credentials: {commands}\n\n"
                        "If your token still does not work, please try to "
                        "clear the cache :\n "
                        "from pynsee.utils import clear_all_cache;"
                        " clear_all_cache()\n"
                    )
                    raise requests.exceptions.RequestException(msg)

            if api_url:
                result = self._request_api_insee(
                    api_url,
                    file_format=file_format,
                    raise_if_not_ok=raise_if_not_ok,
                    print_msg=print_msg,
                )
            elif sdmx_url:
                result = self._request_sdmx_insee(
                    sdmx_url, raise_if_not_ok=raise_if_not_ok
                )
            else:
                raise ValueError(
                    "either api_url or sdmx_url must be set: urls are "
                    "currently missing"
                )

        except requests.exceptions.RequestException:
            if sdmx_url:
                if print_msg:
                    logger.critical("SDMX web service used instead of API")
                result = self._request_sdmx_insee(
                    sdmx_url, raise_if_not_ok=raise_if_not_ok
                )
            else:
                # in that case, simply raise the exception
                raise

        return result

    def _query_is_valid_sirene_call(self, url: str) -> bool:
        """
        Check if an URL is a valid API call to SIRENE.

        Parameters
        ----------
        url : str
            Url to check.

        Returns
        -------
        bool
            True if the API is a SIRENE api query. False if not.

        """
        return bool(re.match(".*api-sirene.*", url))

    def _request_sdmx_insee(
        self, url: str, raise_if_not_ok: bool = True
    ) -> requests.Response:
        """
        Performs a "get" query on the SDMX (Statistical Data and Metadata
        eXchange) webservice of INSEE.

        Parameters
        ----------
        url : str
            URL to query.
        raise_if_not_ok : bool, optional
            If set to True, a RequestException will automatically be raised if
            the response is not ok (= `status_code` < 400).
            The default is True.

        Raises
        ------
        RequestException
            If the requests fails (only if raise_if_not_ok is set to True).

        Returns
        -------
        results : requests.Response
            HTTP response from the requests package.

        """

        results = self.get(url, verify=False)
        if raise_if_not_ok and not results.ok:
            raise requests.exceptions.RequestException(
                results.text + "\n" + url, response=results
            )
        return results

    def _request_api_insee(
        self,
        url: str,
        file_format: str = "application/xml",
        raise_if_not_ok: bool = False,
        print_msg: bool = True,
    ) -> requests.Response:
        """
        Performs a "get" query on the INSEE's API portal.

        Parameters
        ----------
        url : str
            URL to query.
        file_format : str, optional
            Which king of file to expect. This currently alters the output of
            INSEE's APIs. The default is "application/xml".
        raise_if_not_ok : bool, optional
            If set to True, a RequestException will automatically be raised if
            the response is not ok (= `status_code` < 400).
            The default is False.
        print_msg : bool, optional
            If True, will log critical entries to warn that the call to
            APIs failed. The default is True.

        Raises
        ------
        RequestException
            In case of requests failure.

        Returns
        -------
        results : requests.Response
            HTTP response from the requests package.

        """

        headers = {"Accept": file_format}

        try:
            results = self.get(
                url,
                headers=headers,
                verify=False,
                raise_if_not_ok=raise_if_not_ok,
            )
        except Exception:
            results = None
            success = False
        else:

            code = results.status_code

            if "status_code" not in dir(results):
                success = False
            elif code == 429:

                display_warnings = os.environ.get(
                    "PYNSEE_DISPLAY_ALL_WARNINGS", ""
                ).lower()
                if display_warnings == "true":
                    msg = (
                        "API query number limit reached - "
                        "function might be slowed down"
                    )
                    logger.warning(msg)

                time.sleep(10)

                request_again = self._request_api_insee(
                    url=url, file_format=file_format
                )

                return request_again

            elif code in self.INSEE_API_CODES and raise_if_not_ok:
                msg = (
                    f"Error {code} - {self.INSEE_API_CODES[code]}\n"
                    f"Query:\n{url}"
                )
                raise requests.exceptions.RequestException(
                    msg, response=results
                )
            elif code not in (200, 404) and raise_if_not_ok:
                # Note : 404 means no results from API, this should not trigger
                # any exception
                success = False
            else:
                success = True

        if success is True:
            return results

        else:

            try:
                results = results.text
            except Exception:
                results = ""

            if print_msg:
                msg = (
                    "An error occurred !\n"
                    f"Query : {url}\nResults : {results}\n"
                )

                logger.warning(msg)
            raise requests.exceptions.RequestException(
                "An error occured", response=results
            )

    def _test_connections(self) -> dict:
        """
        Test the valid connection to each API.

        Raises
        ------
        RequestException
            If we got a proxy error or a 404 exception.
        ValueError
            If no API was reached.

        Returns
        -------
        invalid_requests : dict
            Dict of {"api name": response.status_code} for invalid queries.

        """
        queries = {
            "BDM": "https://api.insee.fr/series/BDM/dataflow/FR1/all",
            "Metadata": "https://api.insee.fr/metadonnees/codes/cj/n3/5599",
            "Sirene": "https://api.insee.fr/api-sirene/3.11/siret?q=activitePrincipaleUniteLegale:86.10*&nombre=1000",
            "Local Data": "https://api.insee.fr/donnees-locales/donnees/geo-SEXE-DIPL_19@GEO2020RP2017/FE-1.all.all",
        }
        invalid_requests = {}

        for api, api_url in queries.items():

            try:
                if api == "Sirene" and not self.sirene_key:
                    # the user is probably not expecting to use SIRENE anyway:
                    # simple warning and jump to next API in order to avoid
                    # urllib retries
                    logger.warning(
                        "Remember to subscribe to SIRENE API on api.insee.fr "
                        "if you ever want to use it (type `help(init_conn)` "
                        "to know more about this)."
                    )
                    invalid_requests[api] = 999

                self.get(api_url, verify=False, stream=True)
            except requests.exceptions.RequestException as exc:
                # raise error only if 404 or not status code was returned
                if not hasattr(exc.response, "status_code"):
                    raise requests.exceptions.RequestException(
                        f"Could not reach {api} at {api_url}, please control "
                        "your proxy configuration "
                        f"- proxies were {self.proxies}."
                    ) from exc
                elif exc.response.status_code == 404:
                    raise requests.exceptions.RequestException(
                        f"Could not reach {api} at {api_url}, the server "
                        "returned 404 (not found); please get in touch if "
                        "the issue persists."
                    ) from exc

                invalid_requests[api] = exc.response.status_code

        if len(invalid_requests) == len(queries):
            raise ValueError(
                "No API was reached. That's strange, please get in touch if "
                "the issue persists."
            )

        return invalid_requests



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import functools


def _reduce_concat(x, sep=""):

    return functools.reduce(lambda x, y: str(x) + sep + str(y), x)


def _paste(*lists, sep=" ", collapse=None):
    result = map(lambda x: _reduce_concat(x, sep=sep), zip(*lists))
    if collapse is not None:
        return _reduce_concat(result, sep=collapse)
    else:
        return list(result)



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

from functools import lru_cache
import pandas as pd
import re


@lru_cache(maxsize=None)
def _get_naf(file):
    def clean_CODE_col(string):
        re_searched = re.search("^SECTION .*$", string)
        if re_searched:
            string_found = re_searched.group(0)
            string_found_cleaned = string_found.replace("SECTION ", "")
        else:
            string_found_cleaned = string
        return string_found_cleaned

    naf = pd.read_csv(
        file,
        sep=";",
        encoding="ISO-8859-1",
        # encoding='latin1'
    )

    # data cleaning
    naf = naf.iloc[:, 1:5]
    naf.columns = ["CODE", "TITLE_FR", "TITLE_65CH_FR", "TITLE_40CH_FR"]
    naf = naf.dropna()

    naf["CODE"] = naf["CODE"].apply(clean_CODE_col)

    naf["len_code"] = naf["CODE"].str.len()
    return naf



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

from functools import lru_cache
import re
import zipfile
import importlib
import pandas as pd

from pynsee.utils._create_insee_folder import _create_insee_folder
from pynsee.utils._paste import _paste
from pynsee.metadata._get_naf import _get_naf
from pynsee.metadata._get_nomenclature_agreg import _get_nomenclature_agreg
from pynsee.metadata._add_A17_activity import _add_A17_activity
from pynsee.metadata._add_A5_activity import (
    _get_A5_activity_label,
    _add_A5_activity,
)

import logging

logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def _warning_activity():
    msg1 = "This function renders package's internal data"
    logger.info(msg1)


@lru_cache(maxsize=None)
def get_activity_list(level):
    """Get a list of economic activities from NAF/NACE rev 2 2008 classification

    Notes:
        This function uses NAF/NACE rev. 2 classification made in 2008. This function renders only package's internal data.

    Args:
        level (str): Levels available are :  A5, A10, A17, A21, A38, A64, A88, A129, A138, NAF1, NAF2, NAF3, NAF4, NAF5

    Raises:
        ValueError: an error is raised if level is not in the default list

    Examples:
        >>> from pynsee.metadata import get_activity_list
        >>> activity_A138 = get_activity_list('A138')
        >>> activity_NAF3 = get_activity_list('NAF3')
        >>> activity_NAF5 = get_activity_list('NAF5')
    """

    level = level.upper()

    level_available = [
        "A5",
        "A10",
        "A17",
        "A21",
        "A38",
        "A64",
        "A88",
        "A129",
        "A138",
        "NAF1",
        "NAF2",
        "NAF3",
        "NAF4",
        "NAF5",
    ]

    if level not in level_available:
        raise ValueError(
            "!!! level must be in %s !!!",
            _paste(level_available, collapse=" "),
        )

    A5_activity_list = _get_A5_activity_label()

    _warning_activity()

    if level == "A5":
        return A5_activity_list

    insee_folder = _create_insee_folder()

    list_expected_files = [
        "int_courts_naf_rev_2.csv",
        "naf2008_5_niveaux.csv",
        "int_eng_na_2008_a10.csv",
        "int_eng_na_2008_a17.csv",
        "int_eng_na_2008_a21.csv",
        "int_eng_na_2008_a38.csv",
        "int_eng_na_2008_a64.csv",
        "int_eng_na_2008_a88.csv",
        "int_eng_na_2008_a138.csv",
        "niv_agreg_naf_rev_2.csv",
        "table_NAF2-NA.csv",
    ]

    list_expected_files = [
        insee_folder + "/naf2008/" + f for f in list_expected_files
    ]

    # unzipping raw files
    try:
        pkg_path = importlib.resources.files(__name__)
        zip_file = str(pkg_path) + "/data/naf.zip"
    except Exception:
        import pkg_resources

        zip_file = pkg_resources.resource_stream(__name__, "data/naf.zip")

    with zipfile.ZipFile(zip_file, "r") as zip_ref:
        zip_ref.extractall(insee_folder)

    def drop_space(string):
        if pd.isna(string):
            return string
        re_searched = re.search(" .*", string)
        if re_searched:
            string_found_cleaned = string.replace(" ", "")
        else:
            string_found_cleaned = string
        return string_found_cleaned

    if level in ["NAF5", "NAF4", "NAF3", "NAF2", "NAF1"]:
        # read files
        naf_level = pd.read_csv(
            list_expected_files[1],
            sep=";",
            encoding="ISO-8859-1",
            # encoding='latin',
            dtype=str,
        )

        naf_level.columns = ["NAF5", "NAF4", "NAF3", "NAF2", "NAF1"]

        naf = _get_naf(file=list_expected_files[0])

        naf = naf.rename(
            columns={
                "CODE": level,
                "TITLE_FR": "TITLE_" + level + "_FR",
                "TITLE_65CH_FR": "TITLE_" + level + "_65CH_FR",
                "TITLE_40CH_FR": "TITLE_" + level + "_40CH_FR",
            }
        )

        mapp = pd.read_csv(
            list_expected_files[10],
            sep=";",
            encoding="ISO-8859-1",
            # encoding='latin',
            dtype=str,
        )

        mapp = mapp.iloc[:, [0] + list(range(2, 10))]
        mapp = mapp[mapp.index != 0]
        # mapp.columns = ['A732', 'A615', 'A272',
        #                 'A129', 'A88', 'A64', 'A38', 'A21', 'A10']

        mapp.columns = [
            "NAF5",
            "NAF4",
            "NAF3",
            "A129",
            "A88",
            "A64",
            "A38",
            "A21",
            "A10",
        ]
        mapp["NAF2"] = mapp["A88"]
        mapp["NAF1"] = mapp["A21"]

        mapp = mapp[[level, "A129", "A88", "A64", "A38", "A21", "A10"]]

        if level == "NAF1":
            naf = naf[naf["len_code"] == 1]
            mapp = mapp[[level, "A21", "A10"]]
        if level == "NAF2":
            naf = naf[naf["len_code"] == 2]
            naf_level = naf_level[["NAF1", "NAF2"]]
            mapp = mapp[[level, "A88", "A64", "A38", "A21", "A10"]]
        if level == "NAF3":
            naf = naf[naf["len_code"] == 4]
            naf_level = naf_level[["NAF1", "NAF2", "NAF3"]]
        if level == "NAF4":
            naf = naf[naf["len_code"] == 5]
            naf_level = naf_level[["NAF1", "NAF2", "NAF3", "NAF4"]]
        if level == "NAF5":
            naf = naf[naf["len_code"] == 6]

        if not level == "NAF1":
            naf_level = naf_level.drop_duplicates()
            naf = naf.merge(naf_level, on=level, how="left")

        mapp = mapp.drop_duplicates()
        naf = naf.drop(columns=["len_code"])
        naf = naf.merge(mapp, on=level, how="left")

        if "A38" in naf.columns:
            naf = _add_A17_activity(naf)

        if "A10" in naf.columns:
            naf = _add_A5_activity(naf)

        # sort columns alphabetically
        naf = naf.reindex(sorted(naf.columns), axis=1).reset_index(drop=True)

        return naf

    #
    df = _get_nomenclature_agreg(file=list_expected_files[9])

    level_origin = level
    if level == "A17":
        level = "A38"

    icol = df.columns.get_loc(level)

    if level == "A10":
        df = df[df["A10"] == df["A138"]]
    else:
        level_before = df.columns[icol - 1]
        df = df[df[level] != df[level_before]]

    df = df.drop_duplicates(subset=level, keep="first")

    if level not in ["A138", "A129", "A88"]:
        level_after = df.columns[icol + 1]
        df = df[df[level] == df[level_after]]

    seq = list(range(0, icol + 1)) + [7]
    df = df.iloc[:, seq]

    df = df.rename(columns={"TITLE": "TITLE_" + level + "_FR"})
    df[level] = df[level].apply(drop_space)

    if "A38" in df.columns:
        df = _add_A17_activity(df).reset_index(drop=True)

    level = level_origin

    if level == "A138":
        ifile = 8
        ncol = 6
        first_col = 1
        label_list_col = [
            "A129",
            "A138",
            "NAF",
            "TITLE_A138_EN",
            "TITLE_A138_FR",
        ]
        col_merged = ["A138", "TITLE_A138_EN"]

    if level == "A129":
        ifile = 8
        ncol = 6
        first_col = 1
        label_list_col = [
            "A129",
            "A138",
            "NAF",
            "TITLE_A129_EN",
            "TITLE_A129_FR",
        ]
        col_merged = ["A129", "TITLE_A129_EN"]

    if level == "A88":
        ifile = 7
        ncol = 5
        first_col = 1
        label_list_col = ["", "A88", "TITLE_A88_EN", "TITLE_A88_FR"]
        col_merged = ["A88", "TITLE_A88_EN"]

    if level == "A64":
        ifile = 6
        ncol = 4
        first_col = 0
        label_list_col = ["A64", "NAF", "TITLE_A64_EN", "TITLE_A64_FR"]
        col_merged = ["A64", "TITLE_A64_EN"]

    if level == "A38":
        ifile = 5
        ncol = 4
        first_col = 0
        label_list_col = ["A38", "NAF", "TITLE_A38_EN", "TITLE_A38_FR"]
        col_merged = ["A38", "TITLE_A38_EN"]

    if level == "A21":
        ifile = 4
        ncol = 4
        first_col = 1
        label_list_col = ["A21", "TITLE_A21_EN", "TITLE_A21_FR"]
        col_merged = ["A21", "TITLE_A21_EN"]

    if level == "A10":
        ifile = 2
        ncol = 5
        first_col = 0
        label_list_col = ["", "A10", "", "TITLE_A10_EN", "TITLE_A10_FR"]
        col_merged = ["A10", "TITLE_A10_EN"]

    if level == "A17":
        ifile = 3

    label = pd.read_csv(
        list_expected_files[ifile], sep=";", encoding="ISO-8859-1", dtype=str
    )

    if level != "A17":
        label = label.iloc[:, first_col:ncol]
        label.columns = label_list_col
        label = label.dropna(how="all")

        label[level] = label[level].apply(drop_space)
        df = df.merge(label[col_merged], on=level, how="left")
    else:
        label = label.iloc[:, [0, 5, 6]]
        label.columns = ["A17", "TITLE_A17_EN", "TITLE_A17_FR"]
        df = df[["A10", "A17"]].reset_index(drop=True).drop_duplicates()
        df = df.merge(label, on="A17", how="left")

    # add A5 activity
    if "A10" in df.columns:
        df = _add_A5_activity(df)

    return df



# -*- coding: utf-8 -*-
"""
Created on Wed Feb  1 13:52:54 2023

@author: thomas.grandjean@developpement-durable.gouv.fr
"""

import pandas as pd

from pynsee.utils.requests_session import PynseeAPISession
from pynsee.utils.save_df import save_df

import logging

logger = logging.getLogger(__name__)


@save_df(day_lapse_max=90)
def get_ascending_area(
    area: str,
    code: str,
    date: str = None,
    type: str = None,
    update: bool = False,
    silent: bool = False,
):
    """
    Get information about areas containing a given area

    Args:
        area (str): case sensitive, area type, any of ('arrondissement', 'arrondissementMunicipal', 'circonscriptionTerritoriale', 'commune', 'communeAssociee', 'communeDeleguee', 'departement', 'district')

        code (str): area code

        type (str) : case insensitive, any of 'Arrondissement', 'Departement', 'Region', 'UniteUrbaine2020', 'ZoneDEmploi2020', ...

        date (str, optional): date used to analyse the data, format : 'AAAA-MM-JJ'. If date is None, by default the current date is used.

        update (bool): locally saved data is used by default. Trigger an update with update=True.

        silent (bool, optional): Set to True to disable messages printed in log info

    Examples:
        >>> from pynsee.localdata import get_ascending_area
        >>> df = get_ascending_area("commune", code='59350', date='2018-01-01')
        >>> df = get_ascending_area("departement", code='59')
    """

    areas = {
        "arrondissement",
        "arrondissementMunicipal",
        "circonscriptionTerritoriale",
        "commune",
        "communeAssociee",
        "communeDeleguee",
        "departement",
        "district",
    }
    if area not in areas:
        msg = f"area must be one of {areas} " f"- found '{area}' instead"
        raise ValueError(msg)

    params_hash = ["get_ascending_area", area, code, date, type]
    params_hash = [x if x else "_" for x in params_hash]

    INSEE_localdata_api_link = "https://api.insee.fr/metadonnees/geo/"

    api_link = INSEE_localdata_api_link + area + f"/{code}/ascendants?"

    params = []
    if date is not None:
        params.append(f"date={date}")
    if type is not None:
        params.append(f"type={type}")

    api_link = api_link + "&".join(params)

    with PynseeAPISession() as session:
        request = session.request_insee(
            api_url=api_link, file_format="application/json"
        )

    try:
        data = request.json()

        list_data = []

        for i in range(len(data)):
            df = pd.DataFrame(data[i], index=[0])
            list_data.append(df)

        data_final = pd.concat(list_data).reset_index(drop=True)

    except Exception:
        logger.error("No data found !")
        data_final = pd.DataFrame()

    return data_final



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

from pynsee.sirene import search_sirene


def get_dimension_list(kind="siret"):
    """Get a list of all columns useful to make queries with search_sirene

    Args:
        kind (str, optional): Choose between siret and siren. Defaults to 'siret'.

    Examples:
        >>> from pynsee.sirene import get_dimension_list
        >>> sirene_dimension = get_dimension_list()
    """

    if kind not in ["siret", "siren"]:
        raise ValueError("kind must be siret (default) or siren")

    if kind == "siret":
        df = search_sirene(
            variable=["sigleUniteLegale"],
            pattern=["INSEE"],
            closed=True,
            kind="siret",
            number=1,
        )
        col_to_keep = 0
    else:
        df = search_sirene(
            variable=["sigleUniteLegale"],
            pattern=["INSEE"],
            kind="siren",
            closed=True,
            number=1,
        )
        col_to_keep = 1

    df = df.T
    df = df.reset_index(drop=False)
    df = df.loc[:, ["index", col_to_keep]]
    df.columns = ["siret_columns", "example"]
    name_first_col = df.columns[0]

    list_added_col = [
        "categorieJuridiqueUniteLegale",
        "activitePrincipaleUniteLegale",
        "activitePrincipaleEtablissement",
        "typeVoieEtablissement",
    ]
    list_added_col = [c + "Libelle" for c in list_added_col]
    list_added_col = list_added_col + [
        "effectifsMinEtablissement",
        "effectifsMaxEtablissement",
    ]

    string_col_remove = ["^" + c + "$" for c in list_added_col]
    string_col_remove = "|".join(string_col_remove)

    df = df[~(df[name_first_col].str.contains(string_col_remove))]
    df = df.reset_index(drop=True)

    return df



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import pandas as pd

from pynsee.metadata.get_activity_list import get_activity_list
from pynsee.metadata.get_legal_entity import get_legal_entity
from pynsee.sirene._employee_metadata import _employee_metadata
from pynsee.sirene._street_metadata import _street_metadata
from pynsee.utils._move_col_after import _move_col_after

import logging

logger = logging.getLogger(__name__)

# @lru_cache(maxsize=None)


def _clean_data(
    data_final,
    kind="siren",
    clean=True,
    activity=True,
    legal=True,
    only_alive=True,
):
    # try:
    # add activity metadata
    if data_final is not None:
        if activity:
            naf5 = get_activity_list("NAF5")
            naf5 = naf5[["NAF5", "TITLE_NAF5_FR"]]

            if "activitePrincipaleUniteLegale" in data_final.columns:
                naf5_merge = naf5.rename(
                    columns={
                        "NAF5": "activitePrincipaleUniteLegale",
                        "TITLE_NAF5_FR": "activitePrincipaleUniteLegaleLibelle",
                    }
                )
                data_final = data_final.merge(
                    naf5_merge, on="activitePrincipaleUniteLegale", how="left"
                )

            if "activitePrincipaleEtablissement" in data_final.columns:
                if (
                    "activitePrincipaleEtablissementLibelle"
                    not in data_final.columns
                ):
                    naf5_merge = naf5.rename(
                        columns={
                            "NAF5": "activitePrincipaleEtablissement",
                            "TITLE_NAF5_FR": "activitePrincipaleEtablissementLibelle",
                        }
                    )
                    data_final = data_final.merge(
                        naf5_merge,
                        on="activitePrincipaleEtablissement",
                        how="left",
                    )

        # remove companies which no longer exist
        if only_alive:
            if "dateFin" in data_final.columns:
                data_final = data_final[data_final["dateFin"].isnull()]
            if "etatAdministratifEtablissement" in data_final.columns:
                data_final = data_final[
                    data_final["etatAdministratifEtablissement"] == "A"
                ]
            if "etatAdministratifUniteLegale" in data_final.columns:
                data_final = data_final[
                    data_final["etatAdministratifUniteLegale"] == "A"
                ]

        # add legal entities title
        if legal:
            if "categorieJuridiqueUniteLegale" in data_final.columns:
                if (
                    "categorieJuridiqueUniteLegaleLibelle"
                    not in data_final.columns
                ):
                    try:
                        list_legal_code = (
                            data_final.categorieJuridiqueUniteLegale.unique()
                        )
                        data_legal = get_legal_entity(
                            list_legal_code, print_err_msg=False
                        )
                        data_legal = data_legal[["code", "title"]]
                        data_legal = data_legal.rename(
                            columns={
                                "code": "categorieJuridiqueUniteLegale",
                                "title": "categorieJuridiqueUniteLegaleLibelle",
                            }
                        )
                        data_final = data_final.merge(
                            data_legal,
                            on="categorieJuridiqueUniteLegale",
                            how="left",
                        )
                    except Exception:
                        pass
        # empty columns at the end
        list_all_na_col = [
            col for col in data_final.columns if all(pd.isna(data_final[col]))
        ]
        list_first_col = [
            col for col in data_final.columns if col not in list_all_na_col
        ]
        data_final = data_final[list_first_col + list_all_na_col]

        # change colummn order
        if kind == "siren":
            first_col = [
                "siren",
                "denominationUniteLegale",
                "dateFin",
                "dateDebut",
                "categorieEntreprise",
                "categorieJuridiqueUniteLegale",
                "activitePrincipaleUniteLegale",
            ]

        if kind == "siret":
            first_col = [
                "siren",
                "nic",
                "siret",
                "dateDebut",
                "dateCreationEtablissement",
                "dateCreationUniteLegale",
                "dateFin",
                "denominationUniteLegale",
                "nomUniteLegale",
                "prenomUsuelUniteLegale",
                "categorieEntreprise",
                "categorieJuridiqueUniteLegale",
                "activitePrincipaleUniteLegale",
                "activitePrincipaleEtablissement",
                "numeroVoieEtablissement",
                "typeVoieEtablissement",
                "libelleVoieEtablissement",
                "codePostalEtablissement",
                "libelleCommuneEtablissement",
                "codeCommuneEtablissement",
            ]

        if set(first_col).issubset(data_final.columns):
            other_col = [col for col in data_final if col not in first_col]
            data_final = data_final[first_col + other_col]

        # add employee range metadata
        df_empl_siren = _employee_metadata(kind="siren")
        df_empl_siret = _employee_metadata(kind="siret")

        if "trancheEffectifsUniteLegale" in data_final.columns:
            if "effectifsMinUniteLegale" not in data_final.columns:
                data_final = data_final.merge(
                    df_empl_siren, on="trancheEffectifsUniteLegale", how="left"
                )

        if "trancheEffectifsEtablissement" in data_final.columns:
            if "effectifsMinEtablissement" not in data_final.columns:
                data_final = data_final.merge(
                    df_empl_siret,
                    on="trancheEffectifsEtablissement",
                    how="left",
                )

        # add street metadata
        df_street = _street_metadata()
        if "typeVoieEtablissement" in data_final.columns:
            if "typeVoieEtablissementLibelle" not in data_final.columns:
                data_final = data_final.merge(
                    df_street, on="typeVoieEtablissement", how="left"
                )

        # move columns title after columns containing values
        for var in [
            "categorieJuridiqueUniteLegale",
            "activitePrincipaleUniteLegale",
            "activitePrincipaleEtablissement",
            "typeVoieEtablissement",
        ]:
            data_final = _move_col_after(data_final, var + "Libelle", var)

        for k in ["Etablissement", "UniteLegale"]:
            data_final = _move_col_after(
                data_final, "effectifsMax" + k, "trancheEffectifs" + k
            )
            data_final = _move_col_after(
                data_final, "effectifsMin" + k, "trancheEffectifs" + k
            )

        if clean:
            data_final = data_final.dropna(axis=1, how="all")

    return data_final



# -*- coding: utf-8 -*-

import pandas as pd


def _make_dataframe_from_dict(idata):

    key_list = [key for key in idata.keys() if isinstance(idata[key], list)]
    key_not_list = [
        key for key in idata.keys() if not isinstance(idata[key], list)
    ]

    key_dict = [key for key in idata.keys() if isinstance(idata[key], dict)]
    key_not_list_dict = [
        key for key in key_not_list if not isinstance(idata[key], dict)
    ]

    data_from_list = []
    for key in key_list:
        df = pd.DataFrame(idata[key])
        data_from_list.append(df)

    newdict = {key: idata[key] for key in key_not_list_dict}

    data_other0 = pd.DataFrame(newdict, index=[0])

    data_from_dict = []
    for key in key_dict:
        df = pd.DataFrame(idata[key], index=[0])
        data_from_dict.append(df)

    if len(data_from_list) != 0:
        data_from_list = pd.concat(data_from_list).reset_index(drop=True)

        list_data_other = []
        for i in range(len(data_from_list.index)):
            list_data_other.append(data_other0)
        data_other = pd.concat(list_data_other).reset_index(drop=True)

        if len(data_from_dict) != 0:
            data_dict0 = pd.concat(data_from_dict, axis=1)
            list_data_dict = []
            for i in range(len(data_from_list.index)):
                list_data_dict.append(data_dict0)
            data_dict = pd.concat(list_data_dict).reset_index(drop=True)
            data_final = pd.concat(
                [data_other, data_dict, data_from_list], axis=1
            )
        else:
            data_final = pd.concat([data_other, data_from_list], axis=1)

    else:
        if len(data_from_dict) != 0:
            data_dict0 = pd.concat(data_from_dict, axis=1)
            data_final = pd.concat([data_other0, data_dict0], axis=1)
        else:
            data_final = data_other0

    return data_final



# -*- coding: utf-8 -*-

from .geofrdataframe import GeoFrDataFrame
from .get_geodata import get_geodata
from .get_geodata_list import get_geodata_list
from .translate_and_zoom import transform_overseas, zoom


__all__ = [
    "GeoFrDataFrame",
    "get_geodata",
    "get_geodata_list",
    "transform_overseas",
    "zoom",
]



import os
import pickle
from shapely.geometry import Polygon, Point

from pynsee.geodata._get_bbox_list_full import _get_bbox_list_full

from pynsee.utils._create_insee_folder import _create_insee_folder
from pynsee.utils._hash import _hash


def _get_bbox_list(polygon=None, update=False, crsPolygon="EPSG:4326"):

    name = "_get_bbox_list"

    if polygon is not None:
        name += "".join([str(i) for i in list(polygon.bounds)])

    insee_folder = _create_insee_folder()
    file_name = insee_folder + "/" + _hash(name)

    bbox = _get_bbox_list_full(crs=crsPolygon)

    if (not os.path.exists(file_name)) | (update is True):

        bbox_list_final = []

        if polygon is not None:

            for i in range(len(bbox)):

                square = [
                    Point(bbox[i][0], bbox[i][1]),
                    Point(bbox[i][2], bbox[i][1]),
                    Point(bbox[i][2], bbox[i][3]),
                    Point(bbox[i][0], bbox[i][3]),
                ]

                poly_bbox = Polygon([[p.x, p.y] for p in square])

                # select only intersect between bbox grid and the polygon
                if polygon.intersects(poly_bbox):

                    try:
                        intersection = polygon.intersection(poly_bbox)
                    except Exception:
                        try:
                            polygon = polygon.buffer(0)
                            intersection = polygon.intersection(poly_bbox)
                        except Exception:
                            pass

                    if hasattr(intersection, "geoms"):

                        list_intersect_bounds = [
                            intersection.geoms[i].bounds
                            for i in range(len(intersection.geoms))
                        ]

                        for p in range(len(list_intersect_bounds)):
                            if list_intersect_bounds[p] not in bbox_list_final:
                                bbox_list_final.append(
                                    list_intersect_bounds[p]
                                )

            if len(bbox_list_final) == 0:
                bbox_list_final = bbox
        else:
            bbox_list_final = bbox

        open_file = open(file_name, "wb")
        pickle.dump(bbox_list_final, open_file)
        open_file.close()

    else:
        try:
            open_file = open(file_name, "rb")
            bbox_list_final = pickle.load(open_file)
            open_file.close()
        except Exception:
            os.remove(file_name)
            bbox_list_final = _get_bbox_list(polygon=polygon, update=True)

    return bbox_list_final



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import pandas as pd


def _employee_metadata(kind="siren"):

    df = pd.DataFrame(
        {
            "trancheEffectifsUniteLegale": [
                "00",
                "01",
                "02",
                "03",
                "11",
                "12",
                "21",
                "22",
                "31",
                "32",
                "41",
                "42",
                "51",
                "52",
                "53",
            ],
            "effectifsMinUniteLegale": [
                0,
                1,
                3,
                6,
                10,
                20,
                50,
                100,
                200,
                250,
                500,
                1000,
                2000,
                5000,
                10000,
            ],
            "effectifsMaxUniteLegale": [
                0,
                2,
                5,
                9,
                19,
                49,
                99,
                199,
                249,
                499,
                999,
                1999,
                4999,
                9999,
                None,
            ],
        }
    )

    if kind == "siret":
        df.columns = [
            "trancheEffectifsEtablissement",
            "effectifsMinEtablissement",
            "effectifsMaxEtablissement",
        ]

    df = df.reset_index(drop=True)

    return df



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os

from platformdirs import user_config_dir

CONFIG_FILE = os.path.join(
    user_config_dir("pynsee", ensure_exists=True), "config.json"
)

SIRENE_KEY = "sirene_key"
HTTP_PROXY_KEY = "http_proxy"
HTTPS_PROXY_KEY = "https_proxy"



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

from functools import lru_cache

import logging

logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def _warning_local_data():
    logger.info(
        "This function renders only package's internal data, "
        "it might not be the most up-to-date.\n"
        "Have a look at api.insee.fr !"
    )



import os
import re
from tqdm import trange
import datetime
import pickle
import pandas as pd

from pynsee.localdata._get_insee_local_onegeo import _get_insee_local_onegeo
from pynsee.utils._create_insee_folder import _create_insee_folder
from pynsee.utils._hash import _hash
from pynsee.utils.HiddenPrints import HiddenPrints

import logging

logger = logging.getLogger(__name__)


def _find_latest_local_dataset(
    dataset_version, variables, nivgeo, codegeo, update, backwardperiod=6
):

    filename = _hash(
        "".join([dataset_version] + ["_find_latest_local_dataset"])
    )
    insee_folder = _create_insee_folder()
    file_localdata = insee_folder + "/" + filename

    if (not os.path.exists(file_localdata)) or update:

        datasetname = dataset_version.replace("latest", "").replace("GEO", "")

        current_year = int(datetime.datetime.today().strftime("%Y"))

        list_geo_dates = range(current_year, current_year - backwardperiod, -1)
        list_data_dates = range(
            current_year, current_year - backwardperiod, -1
        )

        if re.compile("^GEOlatest.*latest$").match(dataset_version):

            list_dataset_version = [
                "GEO" + str(gdate) + datasetname + str(ddate)
                for gdate in list_geo_dates
                for ddate in list_data_dates
            ]
        else:
            list_dataset_version = [
                datasetname + str(ddate) for ddate in list_data_dates
            ]

        for dvindex in trange(
            len(list_dataset_version), desc="Finding Latest Dataset Version"
        ):

            dv = list_dataset_version[dvindex]

            try:
                with HiddenPrints():
                    df = _get_insee_local_onegeo(
                        variables, dv, nivgeo=nivgeo, codegeo=codegeo
                    )

                if isinstance(df, pd.DataFrame):
                    if len(df.index) == 1:
                        if df["OBS_VALUE"][0] is None:
                            raise ValueError("No data found")

            except Exception:
                if dv == list_dataset_version[-1]:
                    msg = (
                        "!!! Latest dataset version not found !!!\n"
                        "Please, consider having a look at api.insee.fr or "
                        "get_local_metadata function"
                    )
                    raise ValueError(msg)
            else:
                dataset_version = dv
                break

        f = open(file_localdata, "wb")
        pickle.dump(str(dataset_version), f)
        f.close()
    else:
        try:
            f = open(file_localdata, "rb")
            dataset_version = pickle.load(f)
            f.close()
        except Exception:
            os.remove(file_localdata)
            dataset_version = _find_latest_local_dataset(
                variables=variables,
                dataset_version=dataset_version,
                update=True,
            )
        else:
            logger.info(
                "Latest dataset version previously found has been used\n"
                "Set update=True to get the most up-to-date data"
            )

    return dataset_version



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import os

import platformdirs


def _clean_insee_folder():

    local_appdata_folder = platformdirs.user_cache_dir()
    insee_folder = os.path.join(local_appdata_folder, "pynsee", "pynsee")

    # delete all files in the folder
    if os.path.exists(insee_folder):
        list_file_insee = os.listdir(insee_folder)
        # exclude directories
        list_file_insee = [
            file
            for file in list_file_insee
            if os.path.isfile(os.path.join(insee_folder, file))
        ]

        if len(list_file_insee) > 0:
            for f in list_file_insee:
                os.remove(insee_folder + "/" + f)



from typing import Optional

from geopandas import GeoDataFrame

from ._get_center import _get_center


def _rescale_geom(
    gdf: GeoDataFrame,
    factor: float,
    center: Optional[tuple[float, float]] = None,
) -> None:
    """Rescale geometries of a GeoDataFrame inplace."""
    center = center or _get_center(gdf)

    gdf.loc[:, gdf.geometry.name] = gdf.geometry.scale(
        xfact=factor,
        yfact=factor,
        zfact=1.0,
        origin=center,
    )



import pandas as pd
import difflib
import logging

from pynsee.download._get_value_label import _get_value_label
from pynsee.download._get_dict_data_source import _get_dict_data_source
from pynsee.utils._move_col_after import _move_col_before

logger = logging.getLogger(__name__)


def get_column_metadata(id):
    """Get metadata about an insee.fr file

    Returns:
        Returns the request dataframe as a pandas object

    Examples:
        >>> from pynsee.download import get_column_metadata
        >>> rp_logement_metadata = get_column_metadata("RP_LOGEMENT_2016")

    """
    id = id.upper()
    dict_data_source = _get_dict_data_source()

    list_keys_with_metadata = [
        d
        for d in dict_data_source.keys()
        if ("val_col" in dict_data_source[d].keys())
        | ("label_col" in dict_data_source[d].keys())
    ]

    if id in dict_data_source.keys():
        suggestions = difflib.get_close_matches(
            id, list_keys_with_metadata, n=1, cutoff=0.8
        )
        if len(suggestions) > 0:
            id_used = suggestions[0]
            if not id == id_used:
                logger.warning(
                    f"Metadata for {id} has not been found, metadata for "
                    f"{id_used} is provided instead"
                )
        else:
            id_used = id
    else:
        id_used = id

    if id_used in dict_data_source.keys():
        dict_data = dict_data_source[id_used]

        if "label_col" in dict_data.keys():
            labels = pd.DataFrame(dict_data["label_col"], index=[0]).T
            labels["column"] = labels.index
            labels.columns = ["column_label_fr", "column"]
            labels = _move_col_before(labels, "column", "column_label_fr")
            labels = labels.reset_index(drop=True)
        else:
            logger.warning("Columns labels not found in metadata")
            labels = None

        val_col = _get_value_label(id_used)

        if val_col is not None:
            list_val_col = []

            for k in val_col.keys():
                val_col_df = pd.DataFrame(val_col[k], index=[0]).T
                val_col_df[k] = val_col_df.index
                val_col_df.columns = ["value" + "_label_fr", "value"]
                val_col_df["value"] = val_col_df["value"].astype(str)
                val_col_df["column"] = k
                list_val_col += [val_col_df]

            if len(list_val_col) > 0:
                concat_list = pd.concat(list_val_col).reset_index(drop=True)
                labels = labels.merge(concat_list, on=["column"], how="left")
                labels = labels[
                    ["column", "value", "value_label_fr", "column_label_fr"]
                ]
                logger.info(
                    "Column-specific metadata has been found for this file"
                )

    else:
        raise ValueError(
            "id not found in file list, please check metadata "
            "from get_file_list"
        )

    return labels



# -*- coding: utf-8 -*-

from .get_series_list import get_series_list
from .get_dataset_list import get_dataset_list
from .get_last_release import get_last_release

from .get_series import get_series
from .get_dataset import get_dataset

from .get_column_title import get_column_title
from .get_series_title import get_series_title
from .search_macrodata import search_macrodata

__all__ = [
    "get_series_list",
    "get_dataset_list",
    "get_last_release",
    "get_series",
    "get_dataset",
    "get_column_title",
    "get_series_title",
    "search_macrodata",
]



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import pandas
import math

from pynsee.macrodata._get_insee import _get_insee
from pynsee.macrodata._add_numeric_metadata import _add_numeric_metadata
from pynsee.macrodata._load_dataset_data import _load_dataset_data

from pynsee.utils._paste import _paste
from pynsee.utils.save_df import save_df


@save_df(day_lapse_max=30)
def get_series(
    *idbanks,
    update=False,
    silent=False,
    metadata=True,
    startPeriod=None,
    endPeriod=None,
    firstNObservations=None,
    lastNObservations=None,
    updatedAfter=None,
):
    """Get data from INSEE series idbank

    Args:
        idbanks (str or list or pd.series) : some idbanks provided by get_idbank_list()

        update (bool, optional): Set to True, to update manually the data
        stored locally on the computer. Defaults to False.

        silent (bool, optional): Set to True, to disable messages printed in log info

        metadata (bool, optional): If True, some metadata is added to the data

        startPeriod (str, optional): start date of the data.

        endPeriod (str, optional): end date of the data.

        firstNObservations (int, optional): get the first N observations for each key series (idbank).

        lastNObservations (int, optional): get the last N observations for each key series (idbank).

        updatedAfter (str, optional): starting point for querying the previous releases (format yyyy-mm-ddThh:mm:ss)

    Returns:
        DataFrame: contains the data, indexed by DATE and sorted by IDBANK

    Examples:
        >>> from pynsee.macrodata import get_series_list, get_series
        >>> # inflation figures in France
        >>> df_idbank = get_series_list("IPC-2015")
        >>> df_idbank = df_idbank.loc[
        >>>                    (df_idbank.FREQ == "M") & # monthly
        >>>                    (df_idbank.NATURE == "INDICE") & # index
        >>>                    (df_idbank.MENAGES_IPC == "ENSEMBLE") & # all kinds of household
        >>>                    (df_idbank.REF_AREA == "FE") & # all France including overseas departements
        >>>                    (df_idbank.COICOP2016.str.match("^[0-9]{2}$"))] # coicop aggregation level
        >>> # get data
        >>> data = get_series(df_idbank.IDBANK)
    """
    INSEE_sdmx_link_idbank = (
        "https://bdm.insee.fr/series/sdmx/data/SERIES_BDM/"
    )
    INSEE_api_link_idbank = (
        "https://api.insee.fr/series/BDM/V1/data/SERIES_BDM/"
    )

    #
    # create the parameters to be added to the query
    #

    parameters = [
        "startPeriod",
        "endPeriod",
        "firstNObservations",
        "lastNObservations",
        "updatedAfter",
    ]

    list_addded_param = []
    for param in parameters:
        if eval(param) is not None:
            list_addded_param.append(param + "=" + str(eval(param)))

    added_param_string = ""
    if len(list_addded_param) > 0:
        added_param_string = "?" + _paste(list_addded_param, collapse="&")

    #
    # make one single list of idbanks
    #

    list_idbank = []

    for id in range(len(idbanks)):
        if isinstance(idbanks[id], list):
            list_idbank = list_idbank + idbanks[id]
        elif isinstance(idbanks[id], pandas.core.series.Series):
            list_idbank = list_idbank + idbanks[id].to_list()
        elif isinstance(idbanks[id], str):
            list_idbank = list_idbank + [idbanks[id]]
        else:
            list_idbank = list_idbank + [idbanks[id]]

    #
    # create the ranges of the queries
    # mutliple queries will be created each with 400 idbanks
    #

    n_idbank = len(list_idbank)
    idbank_limit = 400
    max_seq_idbank = math.ceil(n_idbank / idbank_limit)

    list_data = []

    for q in range(max_seq_idbank):
        min_range = q * idbank_limit
        max_range = min((q + 1) * idbank_limit, n_idbank + 1)

        list_idbank_q = list_idbank[min_range:max_range]

        sdmx_query = INSEE_sdmx_link_idbank + _paste(
            list_idbank_q, collapse="+"
        )
        api_query = INSEE_api_link_idbank + _paste(
            list_idbank_q, collapse="%2B"
        )

        if len(list_addded_param) > 0:
            sdmx_query = sdmx_query + added_param_string
            api_query = api_query + added_param_string

        df = _get_insee(
            api_query=api_query,
            sdmx_query=sdmx_query,
            step=str("{0}/{1}").format(q + 1, max_seq_idbank),
        )

        list_data.append(df)

    data = pandas.concat(list_data).reset_index(drop=True)

    if metadata:
        try:
            metadata_df = _load_dataset_data()

            if metadata_df is not None:
                metadata_df = metadata_df.rename(columns={"idbank": "IDBANK"})

                list_idbank_data = list(data.IDBANK.unique())
                metadata_df = metadata_df[
                    metadata_df["IDBANK"].isin(list_idbank_data)
                ].reset_index(drop=True)

                list_col = ["IDBANK"] + [
                    c for c in metadata_df.columns if c not in data
                ]
                metadata_df = metadata_df[list_col]

                data = data.merge(metadata_df, on="IDBANK", how="left")

                # remove all na columns
                data = data.dropna(axis=1, how="all")
        except Exception:
            pass

        try:
            data = _add_numeric_metadata(data)
        except Exception:
            pass

    return data



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import pandas as pd
from tqdm import trange

from .get_dataset_list import get_dataset_list
from ._get_dimensions import _get_dataset_dimension, _get_dimension_values


def get_column_title(dataset=None, update=True):
    """Get the title of a dataset's columns

    Args:
        dataset (str, optional): An INSEE dataset name. Defaults to None, this returns all columns.

    Raises:
        ValueError: Only one string (length one)
        ValueError: Dataset must belong to INSEE datasets list

    Examples:
        >>> from pynsee.macrodata import get_column_title
        >>> insee_all_columns = get_column_title()
        >>> balance_paiements_columns = get_column_title("BALANCE-PAIEMENTS")
    """

    insee_dataset = get_dataset_list()
    insee_dataset_list = insee_dataset["id"].to_list()

    if dataset is None:
        dataset_list = insee_dataset_list
    else:
        for dt in dataset:
            if dt not in insee_dataset_list:
                raise ValueError("%s is not a dataset from INSEE" % dt)
        dataset_list = dataset

    # make a list of all columns
    list_column = []

    n_dataset = len(dataset_list)

    for idt in trange(n_dataset, desc="1/2 - Getting columns list "):
        dt = dataset_list[idt]
        dataset_dimension = _get_dataset_dimension(dt, update=update)
        dataset_dimension = dataset_dimension[
            ["dimension", "local_representation"]
        ]
        list_column.append(dataset_dimension)

    df_column = pd.concat(list_column)
    df_column = df_column.drop_duplicates()

    list_column = []
    n_dimensions = len(df_column.index)

    for irow in trange(n_dimensions, desc="2/2 - Getting values "):
        dim_id = df_column["dimension"].iloc[irow]
        dim_id = str(dim_id).replace("-", "_")
        dim_local_rep = df_column["local_representation"].iloc[irow]

        dim_values = _get_dimension_values(dim_local_rep)

        # drop dimension label
        dim_values = dim_values[dim_values["id"] == dim_local_rep]

        # new column with the dimension id
        dim_values = dim_values.assign(
            column=pd.Series(
                dim_id * len(dim_values.index), index=dim_values.index
            ).values
        )
        dim_values = dim_values[["column", "name_fr", "name_en"]]
        list_column.append(dim_values)

    df_column_final = pd.concat(list_column).reset_index(drop=True)
    return df_column_final



# -*- coding: utf-8 -*-
from functools import lru_cache

import logging

logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def _warning_cached_data(file):
    logger.info(
        f"Previously saved data has been used:\n{file}\n"
        "Set update=True to get the most up-to-date data"
    )



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

from functools import lru_cache

from pynsee.macrodata._get_idbank_internal_data_harmonized import (
    _get_idbank_internal_data_harmonized,
)
from pynsee.macrodata._get_idbank_internal_data import (
    _get_idbank_internal_data,
)
from pynsee.macrodata.get_series_list import get_series_list


@lru_cache(maxsize=None)
def search_macrodata(pattern=".*", metadata=True):
    """Search a pattern among insee series (idbanks) from BDM macroeconomic database

    Notes:
        This function uses package's internal data which might not be the most up-to-date.

    Args:
        pattern (str, optional): String used to filter the idbank list. Defaults to ".*", returns all series.

    Examples:
        >>> from pynsee.macrodata import search_macrodata
        >>> search_all = search_macrodata()
        >>> search_paper = search_macrodata("pâte à papier")
        >>> search_paris = search_macrodata("PARIS")
        >>> search_survey_gdp = search_macrodata("Survey|GDP")
    """

    if pattern not in [".*", ""]:
        idbank_list = _get_idbank_internal_data_harmonized()

        pattern = str(pattern).lower()

        idbank_selected = idbank_list.loc[
            idbank_list.title_en_lower.str.contains(pattern)
            | idbank_list.title_fr_lower.str.contains(pattern)
            | idbank_list.title_fr_no_accent.str.contains(pattern)
        ]

    else:
        idbank_selected = _get_idbank_internal_data()

    idbank_selected = idbank_selected[
        ["DATASET", "IDBANK", "KEY", "TITLE_FR", "TITLE_EN"]
    ]

    if metadata:
        try:
            list_dataset = list(idbank_selected.DATASET.unique())
            metata_df = get_series_list(list_dataset)
            newcol = [
                col
                for col in metata_df.columns
                if col not in idbank_selected.columns
            ] + ["IDBANK"]
            metata_df = metata_df[newcol]

            idbank_selected = idbank_selected.merge(
                metata_df, on="IDBANK", how="left"
            )
            # remove all na columns
            idbank_selected = idbank_selected.dropna(axis=1, how="all")
        except Exception:
            pass

    return idbank_selected



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import pandas as pd
from functools import lru_cache
import re

from pynsee.utils.requests_session import PynseeAPISession
from pynsee.utils._make_dataframe_from_dict import _make_dataframe_from_dict
from pynsee.utils.HiddenPrints import HiddenPrints

from .sirenedataframe import SireneDataFrame

import logging


logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def _warning_get_data():
    logger.warning(
        "This function may return personal data, please check and comply "
        "with the legal framework relating to personal data protection !"
    )


def get_sirene_data(*id):
    """Get data about one or several companies from siren or siret identifiers

    Notes:
        This function may return personal data, please check and comply with the legal framework relating to personal data protection

    Examples:
        >>> from pynsee.sirene import get_sirene_data
        >>> df = get_sirene_data("552081317", "32227167700021")
        >>> df = get_sirene_data(['32227167700021', '26930124800077'])
    """

    list_ids = []

    for i in range(len(id)):
        if isinstance(id[i], list):
            list_ids += id[i]
        elif isinstance(id[i], pd.core.series.Series):
            list_ids += id[i].to_list()
        elif isinstance(id[i], str):
            list_ids += [id[i]]
        else:
            list_ids += [str(id[i])]

    list_data = []

    for i in range(len(list_ids)):
        for kind in ["siret", "siren"]:
            if kind == "siren":
                main_key = "uniteLegale"
            elif kind == "siret":
                main_key = "etablissement"

            INSEE_api_sirene = "https://api.insee.fr/api-sirene/3.11/" + kind
            link = (
                INSEE_api_sirene + "/" + re.sub(r"\s+", "", str(list_ids[i]))
            )

            try:
                with HiddenPrints():
                    with PynseeAPISession() as session:
                        request = session.request_insee(
                            api_url=link,
                            file_format="application/json;charset=utf-8",
                            raise_if_not_ok=True,
                        )

                    data_request = request.json()
                try:
                    data = data_request[main_key]
                except Exception:
                    main_key_list = [
                        key
                        for key in list(data_request.keys())
                        if key != "header"
                    ]
                    main_key = main_key_list[0]
                    data = data_request[main_key]

                data_final = _make_dataframe_from_dict(data)
            except Exception:
                pass
            else:
                list_data.append(data_final)
                break

    if len(list_data) > 0:
        data_final = pd.concat(list_data).reset_index(drop=True)
    else:
        raise ValueError("!!! No data found for the provided identifiers !!!")

    _warning_get_data()

    SireneDF = SireneDataFrame(data_final)

    return SireneDF



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import pandas as pd
import datetime
from functools import lru_cache

from pynsee.utils.requests_session import PynseeAPISession

import logging

logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def _warning_get_new_city():
    logger.info(
        "date is None, by default it is supposed to be ten years before "
        "current year"
    )


@lru_cache(maxsize=None)
def get_new_city(code, date=None):
    """Get data about the new city made from the old ones

    Notes:
        Local data is always provided with a cities classification which depends on a year.
        This classification evolves over time due to the merger of some cities.
        It is often useful to keep track of these mergers to reconcile some data.

        To get a city at a given date, use get_area_projection instead.

    Args:
        code (str): city code

        date (str, optional): date used to analyse the data, format : 'AAAA-MM-JJ'. If date is None, by default it supposed to be ten years before current year.

    Examples:
        >>> from pynsee.localdata import get_next_city
        >>> df = get_next_city(code = '24431', date = '2018-01-01')
    """
    INSEE_localdata_api_link = "https://api.insee.fr/metadonnees/geo/"

    api_link = INSEE_localdata_api_link + "commune/" + str(code) + "/suivants"

    if date is not None:
        api_link = api_link + "?date=" + date
    else:
        _warning_get_new_city()

        now = datetime.datetime.now()
        date = str(now.year - 10)
        api_link = api_link + "?date=" + date + "-01-01"

    with PynseeAPISession() as session:
        request = session.request_insee(
            api_url=api_link, file_format="application/json"
        )

    try:
        data = request.json()

        list_data = []

        for i in range(len(data)):
            df = pd.DataFrame(data[i], index=[0])
            list_data.append(df)

        data_final = pd.concat(list_data).reset_index(drop=True)

    except Exception:
        logger.error("No data found !")
        data_final = pd.DataFrame()

    return data_final



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import io
import zipfile
import importlib
import pandas as pd

from pynsee.utils.save_df import save_df

import logging

logger = logging.getLogger(__name__)


@save_df(day_lapse_max=90)
def _get_idbank_internal_data(update=False, silent=False):

    try:
        pkg_macrodata = importlib.resources.files(__name__)
        zip_file = str(pkg_macrodata) + "/data/idbank_list_internal.zip"
    except Exception:
        import pkg_resources

        zip_file = pkg_resources.resource_stream(
            __name__, "data/idbank_list_internal.zip"
        )

    with zipfile.ZipFile(zip_file, "r") as zip_ref:
        data_file = io.BytesIO(zip_ref.read("idbank_list_internal.csv"))

    idbank_list = pd.read_csv(
        data_file, encoding="utf-8", quotechar='"', sep=",", dtype=str
    )

    col = "Unnamed: 0"
    if col in idbank_list.columns:
        idbank_list = idbank_list.drop(columns={col})

    return idbank_list



from .get_area_list import get_area_list
from .get_geo_list import get_geo_list
from .get_local_data import get_local_data
from .get_nivgeo_list import get_nivgeo_list
from .get_local_metadata import get_local_metadata
from .get_population import get_population
from .get_old_city import get_old_city
from .get_new_city import get_new_city
from .get_ascending_area import get_ascending_area
from .get_descending_area import get_descending_area
from .get_area_projection import get_area_projection

__all__ = [
    "get_area_list",
    "get_geo_list",
    "get_local_data",
    "get_nivgeo_list",
    "get_local_metadata",
    "get_population",
    "get_old_city",
    "get_new_city",
    "get_ascending_area",
    "get_descending_area",
    "get_area_projection",
]



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

from functools import lru_cache
import itertools
import re
from unidecode import unidecode

from pynsee.utils.save_df import save_df
from ._clean_data import _clean_data
from ._request_sirene import _request_sirene
from .sirenedataframe import SireneDataFrame

import logging

logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def _warning_search_sirene():
    logger.warning(
        "This function may return personal data, please check and comply with "
        "the legal framework relating to personal data protection !"
    )


@lru_cache(maxsize=None)
def _warning_data_save():
    logger.info(
        "Locally saved data has been used\n"
        "Set update=True to trigger an update"
    )


@save_df(day_lapse_max=30, cls=SireneDataFrame)
def search_sirene(
    variable,
    pattern,
    kind="siret",
    phonetic_search=False,
    and_condition=True,
    upper_case=False,
    decode=False,
    number=1000,
    activity=True,
    legal=False,
    closed=False,
    update=False,
    silent=False,
):
    """Get data about companies from criteria on variables

    Args:
        variable (str or list): name of the variable on which the search is applied.

        pattern (str or list): the pattern or criterium searched

        kind (str, optional): kind of companies : siren or siret. Defaults to "siret"

        phonetic_search (bool, or list of bool, optional): If True phonetic search is triggered on the all variables of the list, if it is a list of True/False, phonetic search is used accordingly on the list of variables

        and_condition (bool, optional): If True, only records meeting all conditions are kept (AND is inserted between the conditions). If False, all records meeting at least one condition are kept (OR is inserted between the conditions).

        number (int, optional): Number of companies searched. Defaults to 1000. If it is above 1000, multiple queries are triggered.

        upper_case (bool, optional): If True, values of argument 'pattern' are converted to upper case and added to the list of searched patterns.

        decode (bool, optional): If True, values of argument 'pattern' are decoded, especially accents are removed and added to the list of searched patterns.

        activity (bool, optional): If True, activty title is added based on NAF/NACE. Defaults to True.

        legal (bool, optional): If True, legal entities title are added

        closed (bool, optional): If False, closed entities are removed from the data and for each legal entity only the last period for which the data is stable is displayed

        silent (bool, optional): Set to True, to disable messages printed in log info

    Notes:
        This function may return personal data, please check and
        comply with the legal framework relating to personal data protection

    Examples:
        >>> from pynsee.metadata import get_activity_list
        >>> from pynsee.sirene import search_sirene
        >>> from pynsee.sirene import get_dimension_list
        >>> #
        >>> # Get available column names, it is useful to design your query with search_sirene
        >>> sirene_dimension = get_dimension_list()
        >>> #
        >>> # Get activity list (NAF rev 2)
        >>> naf5 = get_activity_list('NAF5')
        >>> #
        >>> # Get a list of hospitals in Paris
        >>> df = search_sirene(variable = ["activitePrincipaleUniteLegale",
        >>>                                "codePostalEtablissement"],
        >>>                    pattern = ["86.10Z", "75*"], kind = "siret")
        >>> #
        >>> # Get a list of companies located in Igny city whose name matches with 'pizza' using a phonetic search
        >>> df = search_sirene(variable = ["libelleCommuneEtablissement",
        >>>                            'denominationUniteLegale'],
        >>>                    pattern = ["igny", 'pizza'],
        >>>                    phonetic_search=True, kind = "siret")
        >>> #
        >>> # Get a list of companies whose name matches with 'SNCF' (French national railway company)
        >>> # and whose legal status is SAS (societe par actions simplifiee)
        >>> df = search_sirene(variable=["denominationUniteLegale",
        >>>                              'categorieJuridiqueUniteLegale'],
        >>>                    pattern=["sncf", '5710'], kind="siren")
        >>> #
        >>> # Get data on Hadrien Leclerc
        >>> df = search_sirene(variable = ['prenom1UniteLegale', 'nomUniteLegale'],
        >>>                           pattern = ['hadrien', 'leclerc'],
        >>>                           phonetic_search = [True, False],
        >>>                           closed=True)
        >>> #
        >>> # Find 2500 tobacco shops
        >>> df = search_sirene(variable = ['denominationUniteLegale'],
        >>>            pattern = ['tabac'],
        >>>            number = 2500,
        >>>            kind = "siret")
        >>> #
        >>> # Find 1000 companies whose name sounds like Dassault Système or is a big company (GE),
        >>> # search is made as well on patterns whose accents have been removed
        >>> import os
        >>> # environment variable 'pynsee_print_url' force the package to print the request
        >>> os.environ["pynsee_print_url"] = 'True'
        >>> df = search_sirene(variable = ["denominationUniteLegale", 'categorieEntreprise'],
        >>>                 pattern = ['Dassot Système', 'GE'],
        >>>                 and_condition = False,
        >>>                 upper_case = True,
        >>>                 decode = True,
        >>>                 update = True,
        >>>                 phonetic_search  = [True, False],
        >>>                 number = 1000)
    """
    if (not kind == "siret") & (not kind == "siren"):
        raise ValueError("!!! kind should be among : siren, siret !!!")

    if not isinstance(phonetic_search, list):
        if phonetic_search is True:
            phntc_list = [True] * len(variable)
        else:
            phntc_list = [False] * len(variable)
    else:
        check_phonetic_search = all(
            isinstance(x, bool) for x in phonetic_search
        )
        if check_phonetic_search is False:
            raise ValueError(
                "!!! phonetic_search must be True, False or a list of True "
                "and False !!!"
            )

        phntc_list = phonetic_search

    alive = not closed

    if isinstance(variable, str):
        variable = [variable]

    if isinstance(pattern, str):
        pattern = [pattern]

    list_siren_hist_variable = [
        "nomUniteLegale",  #
        "nomUsageUniteLegale",  #
        "denominationUniteLegale",  #
        "denominationUsuelle1UniteLegale",  #
        "denominationUsuelle2UniteLegale",  #
        "denominationUsuelle3UniteLegale",
        "categorieJuridiqueUniteLegale",  #
        "etatAdministratifUniteLegale",  #
        "nicSiegeUniteLegale",  #
        "activitePrincipaleUniteLegale",  #
        "caractereEmployeurUniteLegale",  #
        "economieSocialeSolidaireUniteLegale",  #
        # "nomenclatureActivitePrincipaleUniteLegale",
    ] + ["societeMissionUniteLegale"]

    list_siret_hist_variable = [
        "denominationUsuelleEtablissement",  #
        "enseigne1Etablissement",  #
        "enseigne2Etablissement",  #
        "enseigne3Etablissement",  #
        "activitePrincipaleEtablissement",  #
        "etatAdministratifEtablissement",  #
        "nomenclatureActiviteEtablissement",  #
        "caractereEmployeurEtablissement",  #
    ]

    if kind == "siren":
        list_hist_variable = list_siren_hist_variable
    else:
        list_hist_variable = list_siret_hist_variable

    list_var_pattern = []

    for var, patt, phntc in zip(variable, pattern, phntc_list):
        if phntc is False:
            phntc_string = ""
        else:
            phntc_string = ".phonetisation"

        # if pattern has several words, split and put mutiple conditions
        # using OR, unless it's a range condition, cf.
        # https://www.sirene.fr/static-resources/htm/sommaire.html
        if not (
            re.match(r"\[\w+\s+TO\s+\w+\]", patt)
            or re.match(r"\{\w+\s+TO\s+\w+\}", patt)
        ):
            patt = re.sub(r"\s+", "|", patt)
        else:
            # only single spaces are accepted in ranges
            patt = re.sub(r"\s+", " ", patt)

        list_patt = patt.split("|")

        if upper_case:
            list_var_patt_maj = [p.upper() for p in list_patt]
            list_patt += list_var_patt_maj

        if decode:
            list_var_decode = [unidecode(p) for p in list_patt]
            list_patt += list_var_decode

        list_patt = list(set(list_patt))

        list_var_patt = []
        for ptt in list_patt:
            if var in list_hist_variable:
                ptt_h = "periode({}{}:{})".format(var, phntc_string, ptt)
            else:
                ptt_h = "{}{}:{}".format(var, phntc_string, ptt)

            list_var_patt += [ptt_h]

        list_var_pattern.append(list_var_patt)

    if and_condition:
        condition = " AND "
    else:
        condition = " OR "

    permutation = list(itertools.product(*list_var_pattern))
    permutation_and = [condition.join(list(x)) for x in permutation]
    permutation_and_parenth = ["(" + x + ")" for x in permutation_and]
    string_query = " OR ".join(permutation_and_parenth)

    query = "?q=" + string_query

    data_final = _request_sirene(query=query, kind=kind, number=number)

    df = _clean_data(
        data_final.copy(),
        kind=kind,
        clean=False,
        activity=activity,
        legal=legal,
        only_alive=alive,
    )

    if df is not None:
        df = df.reset_index(drop=True)

    _warning_search_sirene()

    return df



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

from pynsee.macrodata._get_insee import _get_insee
from pynsee.macrodata.get_dataset_list import get_dataset_list
from pynsee.macrodata._add_numeric_metadata import _add_numeric_metadata
from pynsee.macrodata.get_series_list import get_series_list

from pynsee.utils._paste import _paste
from pynsee.utils.save_df import save_df


@save_df(day_lapse_max=30)
def get_dataset(
    dataset,
    update=False,
    silent=False,
    metadata=True,
    filter=None,
    startPeriod=None,
    endPeriod=None,
    firstNObservations=None,
    lastNObservations=None,
    updatedAfter=None,
):
    """Get dataset's data from INSEE BDM database

    Args:
        dataset (str): an INSEE dataset included in the list provided by get_dataset_list()

        update (bool, optional): Set to True, to update manually the data stored locally on the computer. Defaults to False.

        silent (bool, optional): Set to True, to disable messages printed in log info

        metadata (bool, optional): If True, some metadata is added to the data

        filter (str, optional): Use the filter to choose only some values in a dimension.
        It is recommended to use it for big datasets. A dimension left empty means
        all values are selected. To select multiple values in one dimension put a "+" between those values.

        startPeriod (str, optional): start date of the data.

        endPeriod (str, optional): end date of the data.

        firstNObservations (int, optional): get the first N observations for each key series (idbank).

        lastNObservations (int, optional): get the last N observations for each key series (idbank).

        updatedAfter (str, optional): starting point for querying the previous releases (format yyyy-mm-ddThh:mm:ss)

    Raises:
        ValueError: dataset should be in INSEE's datasets list

    Returns:
        DataFrame: contains the data

    Examples:
        >>> from pynsee.macrodata import get_dataset
        >>> ipc_data = get_dataset("IPC-2015",
        >>>        filter = "M......ENSEMBLE...CVS.2015",
        >>>        updatedAfter = "2017-07-11T08:45:00")
        >>> #
        >>> business_climate = get_dataset("CLIMAT-AFFAIRES", lastNObservations = 1)
    """

    insee_dataset = get_dataset_list(silent=silent)
    insee_dataset_list = insee_dataset["id"].to_list()

    # check if the dataset exists in INSEE's list
    if dataset not in insee_dataset_list:
        raise ValueError("%s is not a dataset from INSEE" % dataset)

    INSEE_sdmx_link_idbank = "https://bdm.insee.fr/series/sdmx/data/"
    INSEE_api_link_idbank = "https://api.insee.fr/series/BDM/V1/data/"

    sdmx_query = INSEE_sdmx_link_idbank + dataset
    api_query = INSEE_api_link_idbank + dataset

    if filter is not None:
        sdmx_query = sdmx_query + "/" + str(filter) + "."
        api_query = api_query + "/" + str(filter) + "."

    parameters = [
        "startPeriod",
        "endPeriod",
        "firstNObservations",
        "lastNObservations",
        "updatedAfter",
    ]

    list_addded_param = []
    for param in parameters:
        if eval(param) is not None:
            list_addded_param.append(param + "=" + str(eval(param)))

    added_param_string = ""
    if len(list_addded_param) > 0:
        added_param_string = "?" + _paste(list_addded_param, collapse="&")
        sdmx_query = sdmx_query + added_param_string
        if filter is None:
            api_query = api_query + "/" + added_param_string
        else:
            api_query = api_query + added_param_string

    # get data
    data = _get_insee(api_query=api_query, sdmx_query=sdmx_query)

    # add metadata
    if metadata:
        try:
            idbank_list = get_series_list(
                dataset, silent=silent, update=update
            )

            newcol = [
                col for col in idbank_list.columns if col not in data.columns
            ] + ["IDBANK"]
            idbank_list = idbank_list[newcol]

            data = data.merge(idbank_list, on="IDBANK", how="left")
            # remove all na columns
            data = data.dropna(axis=1, how="all")
        except Exception:
            pass

        try:
            data = _add_numeric_metadata(data)
        except Exception:
            pass

    return data



from pynsee.utils._move_col_after import _move_col_after


def _add_A17_activity(df):

    df2 = df.copy()
    df2["A17"] = df2["A38"].copy()

    string = "|".join(["BZ", "DZ", "EZ"])
    df2["A17"] = df2["A17"].replace(to_replace=string, value="DE", regex=True)

    df2["A17"] = df2["A17"].replace(to_replace="CA", value="C1", regex=True)

    df2["A17"] = df2["A17"].replace(to_replace="CD", value="C2", regex=True)

    string = "|".join(["CI", "CJ", "CK"])
    df2["A17"] = df2["A17"].replace(to_replace=string, value="C3", regex=True)

    df2["A17"] = df2["A17"].replace(to_replace="CL", value="C4", regex=True)

    string = "|".join(["CB", "CC", "CE", "CF", "CG", "CH", "CM"])
    df2["A17"] = df2["A17"].replace(to_replace=string, value="C5", regex=True)

    df2["A17"] = df2["A17"].replace(
        to_replace="JA|JB|JC", value="JZ", regex=True
    )

    string = "|".join(["MA", "MB", "MC", "NZ"])
    df2["A17"] = df2["A17"].replace(to_replace=string, value="MN", regex=True)

    string = "|".join(["OZ", "PZ", "QA", "QB"])
    df2["A17"] = df2["A17"].replace(to_replace=string, value="OQ", regex=True)

    string = "|".join(["RZ", "SZ", "TZ", "UZ"])
    df2["A17"] = df2["A17"].replace(to_replace=string, value="RU", regex=True)

    df2 = _move_col_after(df2, "A17", "A10")

    return df2



from pynsee.download._get_dict_data_source import _get_dict_data_source


def _get_value_label(id):
    dict_data_source = _get_dict_data_source()
    val_col = None

    if id in dict_data_source.keys():
        dict_data = dict_data_source[id]

        if "val_col" in dict_data.keys():
            val_col = dict_data["val_col"]

    return val_col



import os
import pandas as pd
from functools import lru_cache
import difflib
import logging

from openpyxl.styles.colors import WHITE, RGB, aRGB_REGEX

from pynsee.download._unzip_pb import _unzip_pb

logger = logging.getLogger(__name__)


def _patch_read_excel_aRGB_hex_values(func):
    """
    patch openpyxl to avoid raising ValueError because of colors styling
    """

    def __patch_set__(self, instance, value):
        if not self.allow_none:
            m = aRGB_REGEX.match(value)
            if m is None:
                value = WHITE
            if len(value) == 6:
                value = "00" + value
        super(RGB, self).__set__(instance, value)

    def wrapper(*args, **kwargs):
        __old_rgb_set__ = RGB.__set__
        try:
            RGB.__set__ = __patch_set__
            return func(*args, **kwargs)
        except Exception:
            raise
        finally:
            RGB.__set__ = __old_rgb_set__

    return wrapper


@lru_cache(maxsize=None)
def warning_file(missingFile, foundFile):
    msgs = [("error", f"Data file missing in the zip file: {missingFile}")]
    if not foundFile == "":
        msgs.append(
            ("error", f"Following file has been used instead: {foundFile}")
        )
    else:
        msgs.append(("warning", "No replacement file has been found"))
    msgs.append(("error", "Please report this issue !"))
    for level, msg in msgs:
        getattr(logger, level)(msg)


@_patch_read_excel_aRGB_hex_values
def _load_data_from_schema(
    telechargementFichier: dict, variables=None, limit_chunk_size=1000000000
):
    """Using options derived from `download_store_file`, import dataset in python

    Arguments:
        telechargementFichier {dict} -- Options needed for import

    Keyword Arguments:
        variables {list} -- A subset of variables that should be used (default: {None})

    Raises:
        ValueError: If the file is not found, an error is raised

    Returns:
        pd.DataFrame -- The required dataset is returned as pd.DataFrame object
    """
    df_insee = None
    file_to_import = telechargementFichier["file_to_import"]

    if telechargementFichier["result"]["zip"] is True:
        zipDirectory = f"{telechargementFichier['import_args']['file']}_temp"

        _unzip_pb(telechargementFichier["file_archive"], f"{zipDirectory}")

        dataFile = telechargementFichier["result"]["fichier_donnees"]
        dataPathFile = os.path.join(f"{zipDirectory}", f"{dataFile}")

        if not os.path.exists(dataPathFile):
            list_file_dir = os.listdir(zipDirectory)

            suggestions = difflib.get_close_matches(
                dataFile, list_file_dir, n=1
            )

            if not len(suggestions) == 0:
                foundFile = suggestions[0]
                file_to_import = os.path.join(
                    f"{zipDirectory}", f"{foundFile}"
                )
            else:
                foundFile = ""

            warning_file(missingFile=dataFile, foundFile=foundFile)
        else:
            file_to_import = dataPathFile

    if os.path.isfile(file_to_import) is False:
        raise ValueError("File cannot be found")

    if telechargementFichier["result"]["type"] == "csv":
        # big file data load
        if os.path.getsize(file_to_import) >= limit_chunk_size:
            list_chunk = []
            chunksize = 10**6
            with pd.read_csv(
                file_to_import,
                chunksize=chunksize,
                dtype="str",
                usecols=variables,
                delimiter=telechargementFichier["import_args"]["delim"],
            ) as reader:
                for chunk in reader:
                    list_chunk += [chunk]

            df_insee = pd.concat(list_chunk)
        else:
            try:
                df_insee = pd.read_csv(
                    file_to_import,
                    delimiter=telechargementFichier["import_args"]["delim"],
                    dtype="str",
                    usecols=variables,
                )
            except Exception:
                encoding = telechargementFichier["import_args"]["encoding"]
                useEncoding = True
                if isinstance(encoding, str):
                    if encoding == "Nan":
                        useEncoding = False
                else:
                    useEncoding = False

                if not useEncoding:
                    df_insee = pd.read_csv(
                        file_to_import,
                        delimiter=telechargementFichier["import_args"][
                            "delim"
                        ],
                        dtype="str",
                        usecols=variables,
                        engine="python",
                    )
                else:
                    df_insee = pd.read_csv(
                        file_to_import,
                        delimiter=telechargementFichier["import_args"][
                            "delim"
                        ],
                        dtype="str",
                        usecols=variables,
                        engine="python",
                        encoding=encoding,
                    )
    elif telechargementFichier["result"]["type"] in ["xls", "xlsx"]:

        df_insee = pd.read_excel(
            file_to_import,
            sheet_name=telechargementFichier["import_args"]["sheet"],
            skiprows=telechargementFichier["import_args"]["skip"],
            nrows=telechargementFichier["import_args"]["n_max"],
            na_values=telechargementFichier["import_args"]["na"],
            dtype="str",
            usecols=variables,
        )
    elif telechargementFichier["result"]["type"] == "parquet":
        df_insee = pd.read_parquet(file_to_import, columns=variables)

    list_files = [file_to_import]

    if telechargementFichier["result"]["zip"] is True:
        list_files += [
            dataPathFile,
            os.path.join(
                f"{zipDirectory}", telechargementFichier["file_archive"]
            ),
            telechargementFichier["file_archive"],
        ]

    for f in list_files:
        if os.path.exists(f):
            try:
                os.remove(f)
            except Exception:
                pass

    if df_insee is None:
        raise RuntimeError(
            f"Unexpected download data:\n{telechargementFichier}"
        )

    return df_insee



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import io
import xml.etree.ElementTree as ET

import pandas as pd

from ..utils.requests_session import PynseeAPISession
from ..utils.save_df import save_df


@save_df(day_lapse_max=90)
def _get_dataset_dimension(
    dataset, update=False, silent=False, insee_date_test=None
):
    """
    Get the variables in the dataset and the correspondance between their
    local and "key" names
    """
    INSEE_sdmx_link_datastructure = (
        "https://bdm.insee.fr/series/sdmx/datastructure/FR1"
    )
    INSEE_api_link_datastructure = (
        "https://api.insee.fr/series/BDM/V1/datastructure/FR1"
    )

    INSEE_sdmx_link_datastructure_dataset = (
        INSEE_sdmx_link_datastructure + "/" + dataset
    )
    INSEE_api_link_datastructure_dataset = (
        INSEE_api_link_datastructure + "/" + dataset
    )

    with PynseeAPISession() as session:
        results = session.request_insee(
            sdmx_url=INSEE_sdmx_link_datastructure_dataset,
            api_url=INSEE_api_link_datastructure_dataset,
        )

    dataset_dimension_file = io.BytesIO(results.content)

    root = ET.parse(dataset_dimension_file).getroot()

    data = root[1][0][0][2][0]

    n_dimension = len(data)

    list_dimension = []

    for i in range(0, n_dimension):
        dimension_id = _extract_id(data, i)
        local_rep = _extract_local_rep(data, i)

        dimension_df = {
            "dataset": dataset,
            "dimension": dimension_id,
            "local_representation": local_rep,
        }

        list_dimension.append(dimension_df)

    dimension_df_all = pd.DataFrame(list_dimension)
    dimension_df_all = dimension_df_all.dropna()

    return dimension_df_all


@save_df(day_lapse_max=90)
def _get_dimension_values(
    cl_dimension, update=False, silent=False, insee_date_test=None
):

    INSEE_sdmx_link_codelist = "https://bdm.insee.fr/series/sdmx/codelist/FR1"
    INSEE_api_link_codelist = "https://api.insee.fr/series/BDM/V1/codelist/FR1"

    INSEE_sdmx_link_codelist_dimension = (
        INSEE_sdmx_link_codelist + "/" + cl_dimension
    )
    INSEE_api_link_codelist_dimension = (
        INSEE_api_link_codelist + "/" + cl_dimension
    )

    with PynseeAPISession() as session:
        results = session.request_insee(
            sdmx_url=INSEE_sdmx_link_codelist_dimension,
            api_url=INSEE_api_link_codelist_dimension,
        )

    dimension_file = io.BytesIO(results.content)

    root = ET.parse(dimension_file).getroot()[1][0]

    list_values = []

    val_id = _extract_id(root, 0)
    name_fr = _extract_name_fr(root, 0)
    name_en = _extract_name_en(root, 0)

    dataset = {"id": [val_id], "name_fr": [name_fr], "name_en": [name_en]}

    dt = pd.DataFrame(dataset, columns=["id", "name_fr", "name_en"])

    list_values.append(dt)

    data = root[0]

    n_values = len(data)

    for i in range(2, n_values):
        val_id = _extract_id(data, i)
        name_fr = _extract_name_fr(data, i)
        name_en = _extract_name_en(data, i)

        dataset = {"id": [val_id], "name_fr": [name_fr], "name_en": [name_en]}

        dt = pd.DataFrame(dataset, columns=["id", "name_fr", "name_en"])

        list_values.append(dt)

    df_dimension_values = pd.concat(list_values)

    return df_dimension_values


def _extract_local_rep(data, i):
    try:
        local_rep = next(iter(data[i][1][0][0].attrib.values()))
    except Exception:
        local_rep = None

    return local_rep


def _extract_id(data, i):
    try:
        id_val = next(iter(data[i].attrib.values()))
    except Exception:
        id_val = None

    return id_val


def _extract_name_fr(data, i):
    try:
        name_fr = data[i][0].text
    except Exception:
        name_fr = None

    return name_fr


def _extract_name_en(data, i):
    try:
        name_en = data[i][1].text
    except Exception:
        name_en = None

    return name_en



import pandas as pd


def _check_df_full_null(df):

    check = all(
        [
            pd.isnull(df.iloc[i, j])
            for i in range(df.shape[0])
            for j in range(df.shape[1])
        ]
    )
    return check



import io
import zipfile
import importlib
import json


def _get_file_list_internal():
    try:
        zip_file = (
            str(importlib.resources.files(__name__))
            + "/data/liste_donnees.zip"
        )
    except Exception:
        import pkg_resources

        zip_file = pkg_resources.resource_stream(
            __name__, "data/liste_donnees.zip"
        )

    with zipfile.ZipFile(zip_file, "r") as zip_ref:
        zip_file = io.BytesIO(zip_ref.read("liste_donnees.json"))

    data = json.load(zip_file)

    return data



from .get_definition_list import get_definition_list
from .get_definition import get_definition
from .get_activity_list import get_activity_list
from .get_legal_entity import get_legal_entity

__all__ = [
    "get_definition_list",
    "get_definition",
    "get_activity_list",
    "get_legal_entity",
]



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

import pandas as pd
from tqdm import trange
import numpy as np
from requests.exceptions import RequestException

from pynsee.utils._paste import _paste
from pynsee.localdata._get_geo_relation import _get_geo_relation
from pynsee.localdata._get_geo_list_simple import _get_geo_list_simple

from pynsee.utils.save_df import save_df


@save_df(day_lapse_max=90)
def get_geo_list(geo=None, date=None, update=False, silent=False):
    """Get a list of French geographic areas (communes, departements, regions ...)

    Args:
        geo (str): choose among : communes, communesDeleguees, communesAssociees, regions, departements, arrondissements, arrondissementsMunicipaux

        date (str): date of validity (AAAA-MM-DD)

        update (bool): locally saved data is used by default. Trigger an update with update=True.

        silent (bool, optional): Set to True, to disable messages printed in log info
    Raises:
        ValueError: geo should be among the geographic area list

    Examples:
        >>> from pynsee.localdata.get_geo_list import get_geo_list
        >>> city_list = get_geo_list('communes')
        >>> region_list = get_geo_list('regions')
        >>> departement_list = get_geo_list('departements')
        >>> arrondiss_list = get_geo_list('arrondissements')
    """

    list_available_geo = [
        "communes",
        "communesDeleguees",
        "communesAssociees",
        "regions",
        "departements",
        "arrondissements",
        "arrondissementsMunicipaux",
    ]
    geo_string = _paste(list_available_geo, collapse=" ")

    error_msg = False

    if geo is None:
        error_msg = True
    elif geo not in list_available_geo:
        error_msg = True

    if error_msg:
        msg = "!!! Please choose geo among:\n{}".format(geo_string)
        raise ValueError(msg)

    RENAME = {
        "Intitule": "TITLE",
        "Type": "TYPE",
        "DateCreation": "DATECREATION",
        "DateSuppression": "DATESUPPRESSION",
        "IntituleSansArticle": "TITLE_SHORT",
        "ChefLieu": "CHEFLIEU",
        "code": "CODE",
        "uri": "URI",
    }

    reg = _get_geo_list_simple("regions", date=date, progress_bar=True)
    reg["DateSuppression"] = reg.get("DateSuppression", np.nan)

    reg = reg.rename(RENAME, axis=1)

    if geo != "regions":
        list_reg = list(reg.CODE)
        list_data_reg = []

        if geo != "arrondissementsMunicipaux":
            type_geo = geo.rstrip("s")
        else:
            type_geo = "ArrondissementMunicipal"

        for r in trange(
            len(list_reg), desc="Getting {}".format(geo), leave=False
        ):
            try:
                df = _get_geo_relation(
                    geo="region",
                    code=list_reg[r],
                    relation="descendants",
                    date=date,
                    type=type_geo,
                )
            except (RequestException, SyntaxError):
                df = _get_geo_relation(
                    geo="region",
                    code=list_reg[r],
                    relation="descendants",
                    date=date,
                    type=None,
                )

            finally:
                list_data_reg.append(df)

        data_all = pd.concat(list_data_reg)

        data_all = data_all.rename(RENAME, axis=1)

        reg_short = reg[["TITLE", "CODE"]]
        reg_short.columns = ["TITLE_REG", "CODE_REG"]
        data_all = data_all.merge(
            reg_short, how="left", left_on="geo_init", right_on="CODE_REG"
        )

        if geo == "communes":
            data_all = data_all.loc[
                data_all["TYPE"].str.match("^Commune$", na=False)
            ]
        if geo == "communesDeleguees":
            data_all = data_all.loc[
                data_all["TYPE"].str.match("^CommuneDeleguee$", na=False)
            ]
        if geo == "communesAssociees":
            data_all = data_all.loc[
                data_all["TYPE"].str.match("^CommuneAssociee$", na=False)
            ]
        if geo == "arrondissements":
            data_all = data_all.loc[
                data_all["TYPE"].str.match("^Arrondissement$", na=False)
            ]
        if geo == "arrondissementsMunicipaux":
            data_all = data_all.loc[
                data_all["TYPE"].str.match(
                    "^ArrondissementMunicipal$", na=False
                )
            ]
        if geo == "departements":
            data_all = data_all.loc[
                data_all["TYPE"].str.match("^Departement$", na=False)
            ]

        try:
            data_all = data_all.drop("geo_init", axis=1)
        except KeyError:
            pass

        if geo != "departements":
            dep = _get_geo_list_simple(
                "departements", date=date, progress_bar=True
            )
            dep = dep.rename(RENAME, axis=1)
            dep_short = dep[["CODE", "TITLE"]]
            dep_short.columns = ["CODE_dep", "TITLE_DEP1"]

            dep_short2 = dep[["CODE", "TITLE"]]
            dep_short2.columns = ["CODE_dep", "TITLE_DEP2"]

            data_all = data_all.assign(
                code_dep1=data_all["CODE"].str[:2],
                code_dep2=data_all["CODE"].str[:3],
            )

            data_all = data_all.merge(
                dep_short,
                how="left",
                left_on="code_dep1",
                right_on="CODE_dep",
            )
            data_all = data_all.merge(
                dep_short2,
                how="left",
                left_on="code_dep2",
                right_on="CODE_dep",
            )

            for i in range(len(data_all.index)):
                if pd.isna(data_all.loc[i, "TITLE_DEP1"]):
                    data_all.loc[i, "CODE_DEP"] = data_all.loc[i, "code_dep2"]
                    data_all.loc[i, "TITLE_DEP"] = data_all.loc[
                        i, "TITLE_DEP2"
                    ]
                else:
                    data_all.loc[i, "CODE_DEP"] = data_all.loc[i, "code_dep1"]
                    data_all.loc[i, "TITLE_DEP"] = data_all.loc[
                        i, "TITLE_DEP1"
                    ]

            data_all = data_all.drop(
                [
                    "code_dep1",
                    "code_dep2",
                    "CODE_dep_x",
                    "CODE_dep_y",
                    "TITLE_DEP1",
                    "TITLE_DEP2",
                ],
                axis=1,
            )

        df_geo = data_all
    else:
        df_geo = reg

    return df_geo



# -*- coding: utf-8 -*-
# Copyright : INSEE, 2021

from functools import lru_cache
import os
import re
import zipfile
import pandas as pd
import importlib

from pynsee.utils._create_insee_folder import _create_insee_folder

import logging

logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def _warning_data():
    logger.info(
        "This function renders only package's internal data, "
        "it might not be the most up-to-date.\n"
        "Have a look at api.insee.fr !"
    )


@lru_cache(maxsize=None)
def get_local_metadata():
    """Get a list of all combinations of datasets, variables and unit measures available from INSEE Local API

    Notes:
        This function renders only package's internal data, it might not be the most up-to-date

    Examples:
        >>> from pynsee.localdata import get_local_metadata
        >>> metadata = get_local_metadata()
    """
    _warning_data()

    insee_folder = _create_insee_folder()

    insee_folder_local_metadata = insee_folder + "/" + "local_metadata"

    os.makedirs(insee_folder_local_metadata, exist_ok=True)

    dataset_label = [
        "Recensement de la population",
        "Séries historiques du recensement de la population (depuis 1968)",
        "Populations légales (issue du RP)",
        "Données de l'état-civil (naissances et décès)",
        "Répertoire des entreprises et des établissements (issu de Sirene)",
        "Fichier localisé social et fiscal",
        "Fichier localisé des rémunérations et de l'emploi salarié",
        "Tourisme (offre d'hébergement)",
    ]
    all_files = []
    name_dataset = [
        "RP",
        "BDCOM",
        "Popleg",
        "RFD",
        "REE",
        "FILOSOFI",
        "Flores",
        "TOUR",
    ]

    var_name = ["var_modalite", "mesure_croisement", "lib_mesure", "millesime"]
    for var in var_name:
        for dt in name_dataset:
            all_files.append("doc_" + dt + "_" + var + ".csv")

    try:
        pkg_path = importlib.resources.files(__name__)
        zip_file = str(pkg_path) + "/data/local_metadata.zip"
    except Exception:
        import pkg_resources

        zip_file = pkg_resources.resource_stream(
            __name__, "data/local_metadata.zip"
        )

    with zipfile.ZipFile(zip_file, "r") as zip_ref:
        zip_ref.extractall(insee_folder)

    def extract_data_from_excel_sheet(
        var,
        list_col,
        reshape=False,
        list_files=all_files,
        folder=insee_folder_local_metadata,
    ):

        list_files = [f for f in list_files if re.search(".*" + var + ".*", f)]

        if reshape is True:
            list_col = ["var" if x == "variable" else x for x in list_col]

        list_var_data = []

        for f in range(len(list_files)):
            try:
                file2load = folder + "/" + list_files[f]
                df = pd.read_csv(file2load, sep=",", encoding="UTF-8")

                if reshape is True:
                    df.columns = [
                        "var" if x == "variable" else x for x in df.columns
                    ]

                    list_other_col = [
                        col for col in df.columns if col not in list_col
                    ]

                    if len(list_other_col) > 0:
                        list_col2 = [
                            col for col in list_col if col in df.columns
                        ]
                        list_col_new2 = list_col2 + ["dataset_value"]
                        # reshape dataframe
                        df = pd.melt(
                            df, id_vars=list_col2, value_vars=list_other_col
                        )
                        # rename col variable into dataset_value
                        df.columns = [
                            "dataset_value" if x == "variable" else x
                            for x in df.columns
                        ]
                        # drop nan in value col
                        df = df[df["value"].notna()]
                        # drop value column
                        df = df[list_col_new2]

                # add column to reference
                file_id = (
                    list_files[f]
                    .replace("doc_", "")
                    .replace(".csv", "")
                    .replace("_" + var, "")
                )
                df = df.assign(dataset=file_id, tab=var)

            except Exception:
                #  logger.info('error {} {}'.format(list_files[f], var))
                pass
            else:
                list_var_data.append(df)

        var_data = pd.concat(list_var_data)
        return var_data

    #
    # get all variables from all datasets
    #

    list_col_mesure_croisement = [
        "mesure",
        "croisement",
        "filtre_stat",
        "filtre_geo",
        "nom_tab",
        "filtre_geo_avt_2017",
        "type_exploitation",
    ]

    mesure_croisement = extract_data_from_excel_sheet(
        var="mesure_croisement",
        list_col=list_col_mesure_croisement,
        reshape=True,
    )

    variables = mesure_croisement[
        ["croisement", "mesure", "dataset_value", "dataset"]
    ]

    #
    # get variables labels
    #
    list_col_var_modalite = ["variable", "lib_var", "modalite", "lib_modalite"]

    var_modalite = extract_data_from_excel_sheet(
        var="var_modalite", list_col=list_col_var_modalite, reshape=True
    )

    var_label = var_modalite[["var", "lib_var"]].drop_duplicates(
        subset="var", keep="first"
    )
    var_label.columns = ["croisement", "variable_label"]

    #
    # add variable labels to variables list
    #
    variables_splitted = variables["croisement"].str.split("-").tolist()

    variables_splitted = pd.DataFrame(
        variables_splitted, index=variables.index
    )

    for icol in range(len(variables_splitted.columns)):
        var_label_icol = var_label
        var_label_icol.columns = [icol, "var_label" + str(icol)]

        variables_splitted = pd.merge(
            variables_splitted, var_label_icol, how="left", on=icol
        )

    var_labels = variables_splitted.filter(regex="var_label")
    var_labels = var_labels.assign(variables_label="")
    icol_var_label = var_labels.columns.get_loc("variables_label")

    for icol in range(len(var_labels.columns) - 1):
        for irow in range(len(var_labels.index)):
            val = var_labels.iloc[irow, icol]
            if not pd.isna(val):
                if var_labels.iloc[irow, icol_var_label] != "":
                    var_labels.iloc[irow, icol_var_label] = (
                        var_labels.iloc[irow, icol_var_label]
                        + " - "
                        + str(val)
                    )
                else:
                    var_labels.iloc[irow, icol_var_label] = str(val)

    var_labels = var_labels[["variables_label"]]
    variables = pd.concat([variables.reset_index(), var_labels], axis=1)

    del variables["index"]

    #
    # add metadata on unit (labels) to variables list
    #
    lib_mesure = extract_data_from_excel_sheet(
        var="lib_mesure", list_col=["mesure", "lib_mesure"]
    )

    lib_mesure = lib_mesure[["mesure", "lib_mesure"]]
    lib_mesure = lib_mesure.drop_duplicates()
    variables = variables.merge(lib_mesure, on="mesure", how="left")

    #
    # add metadata on millesime to variables list: geo data date and data date
    #
    millesime = extract_data_from_excel_sheet(
        var="millesime",
        list_col=["jeu_donnees", "millesime_donnees", "millesime_geo"],
    )

    millesime = millesime[
        ["jeu_donnees", "millesime_geo", "millesime_donnees"]
    ]
    millesime.columns = ["dataset_value", "millesime_geo", "millesime_donnees"]
    millesime = millesime.drop_duplicates()

    variables = variables.merge(millesime, on="dataset_value", how="left")

    dataset_dict = {"dataset": name_dataset, "dataset_label": dataset_label}
    datasets = pd.DataFrame(dataset_dict)
    variables = variables.merge(datasets, on="dataset", how="left")

    variables.columns = [
        "VARIABLES",
        "UNIT",
        "DATASET_VERSION",
        "DATASET",
        "VARIABLES_label_fr",
        "UNIT_label_fr",
        "GEO_DATE",
        "DATA_DATE",
        "DATASET_label_fr",
    ]

    # variables = variables.dropna(subset=['GEO_DATE'])
    variables = variables[
        ~variables.DATASET_VERSION.str.contains("filtre_geo ")
    ]
    variables = variables.reset_index(drop=True)

    return variables
